"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.canUseMultipleAgentPolicies = canUseMultipleAgentPolicies;
exports.canUseOutputForIntegration = canUseOutputForIntegration;
exports.mapPackagePolicySavedObjectToPackagePolicy = void 0;
exports.preflightCheckPackagePolicy = preflightCheckPackagePolicy;
var _constants = require("../../../common/constants");
var _output_helpers = require("../../../common/services/output_helpers");
var _errors = require("../../errors");
var _license = require("../license");
var _output = require("../output");
var _app_context = require("../app_context");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const mapPackagePolicySavedObjectToPackagePolicy = ({
  id,
  version,
  attributes,
  namespaces
}) => {
  return {
    id,
    version,
    spaceIds: namespaces,
    ...attributes
  };
};
exports.mapPackagePolicySavedObjectToPackagePolicy = mapPackagePolicySavedObjectToPackagePolicy;
async function preflightCheckPackagePolicy(soClient, packagePolicy, packageInfo) {
  // Package policies cannot be created for content type packages
  if ((packageInfo === null || packageInfo === void 0 ? void 0 : packageInfo.type) === 'content') {
    throw new _errors.PackagePolicyContentPackageError('Cannot create policy for content only packages');
  }

  // If package policy has multiple agent policies IDs, or no agent policies (orphaned integration policy)
  // check if user can use multiple agent policies feature
  const {
    canUseReusablePolicies,
    errorMessage: canUseMultipleAgentPoliciesErrorMessage
  } = canUseMultipleAgentPolicies();
  if (!canUseReusablePolicies && packagePolicy.policy_ids.length !== 1) {
    throw new _errors.PackagePolicyMultipleAgentPoliciesError(canUseMultipleAgentPoliciesErrorMessage);
  }

  // If package policy has an output_id, check if it can be used
  if (packagePolicy.output_id && packagePolicy.package) {
    const {
      canUseOutputForIntegrationResult,
      errorMessage: outputForIntegrationErrorMessage
    } = await canUseOutputForIntegration(soClient, packagePolicy.package.name, packagePolicy.output_id);
    if (!canUseOutputForIntegrationResult && outputForIntegrationErrorMessage) {
      throw new _errors.PackagePolicyOutputError(outputForIntegrationErrorMessage);
    }
  }
}
function canUseMultipleAgentPolicies() {
  const hasEnterpriseLicence = _license.licenseService.hasAtLeast(_constants.LICENCE_FOR_MULTIPLE_AGENT_POLICIES);
  const {
    enableReusableIntegrationPolicies
  } = _app_context.appContextService.getExperimentalFeatures();
  return {
    canUseReusablePolicies: hasEnterpriseLicence && enableReusableIntegrationPolicies,
    errorMessage: !hasEnterpriseLicence ? 'Reusable integration policies are only available with an Enterprise license' : 'Reusable integration policies are not supported'
  };
}
async function canUseOutputForIntegration(soClient, packageName, outputId) {
  const hasAllowedLicense = _license.licenseService.hasAtLeast(_constants.LICENCE_FOR_OUTPUT_PER_INTEGRATION);
  if (!hasAllowedLicense) {
    return {
      canUseOutputForIntegrationResult: false,
      errorMessage: `Output per integration is only available with an ${_constants.LICENCE_FOR_OUTPUT_PER_INTEGRATION} license`
    };
  }
  const allowedOutputTypes = (0, _output_helpers.getAllowedOutputTypesForIntegration)(packageName);
  const output = await _output.outputService.get(soClient, outputId);
  if (!allowedOutputTypes.includes(output.type)) {
    return {
      canUseOutputForIntegrationResult: false,
      errorMessage: `Output type "${output.type}" is not usable with package "${packageName}"`
    };
  }
  return {
    canUseOutputForIntegrationResult: true,
    errorMessage: null
  };
}