"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.registerGetAllRoute = registerGetAllRoute;
exports.registerGetOneRoute = registerGetOneRoute;
var _configSchema = require("@kbn/config-schema");
var _data_stream_serialization = require("../../../lib/data_stream_serialization");
var _ = require("..");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const enhanceDataStreams = ({
  dataStreams,
  dataStreamsStats,
  meteringStats,
  dataStreamsPrivileges,
  globalMaxRetention,
  indexTemplates
}) => {
  return dataStreams.map(dataStream => {
    const enhancedDataStream = {
      ...dataStream,
      ...(globalMaxRetention ? {
        global_max_retention: globalMaxRetention
      } : {}),
      privileges: {
        delete_index: dataStreamsPrivileges ? dataStreamsPrivileges.index[dataStream.name].delete_index : true,
        manage_data_stream_lifecycle: dataStreamsPrivileges ? dataStreamsPrivileges.index[dataStream.name].manage_data_stream_lifecycle : true
      }
    };
    if (dataStreamsStats) {
      const currentDataStreamStats = dataStreamsStats.find(({
        data_stream: statsName
      }) => statsName === dataStream.name);
      if (currentDataStreamStats) {
        enhancedDataStream.store_size = currentDataStreamStats.store_size;
        enhancedDataStream.store_size_bytes = currentDataStreamStats.store_size_bytes;
        enhancedDataStream.maximum_timestamp = currentDataStreamStats.maximum_timestamp;
      }
    }
    if (meteringStats) {
      const datastreamMeteringStats = meteringStats.find(s => s.name === dataStream.name);
      if (datastreamMeteringStats) {
        enhancedDataStream.metering_size_in_bytes = datastreamMeteringStats.size_in_bytes;
        enhancedDataStream.metering_doc_count = datastreamMeteringStats.num_docs;
      }
    }
    if (indexTemplates) {
      const indexTemplate = indexTemplates.find(template => template.name === dataStream.template);
      if (indexTemplate) {
        var _indexTemplate$index_, _indexTemplate$index_2, _indexTemplate$index_3, _indexTemplate$index_4;
        enhancedDataStream.index_mode = (_indexTemplate$index_ = indexTemplate.index_template) === null || _indexTemplate$index_ === void 0 ? void 0 : (_indexTemplate$index_2 = _indexTemplate$index_.template) === null || _indexTemplate$index_2 === void 0 ? void 0 : (_indexTemplate$index_3 = _indexTemplate$index_2.settings) === null || _indexTemplate$index_3 === void 0 ? void 0 : (_indexTemplate$index_4 = _indexTemplate$index_3.index) === null || _indexTemplate$index_4 === void 0 ? void 0 : _indexTemplate$index_4.mode;
      }
    }
    return enhancedDataStream;
  });
};
const getDataStreams = (client, name = '*') => {
  return client.asCurrentUser.indices.getDataStream({
    name,
    expand_wildcards: 'all'
  });
};
const getDataStreamLifecycle = (client, name) => {
  return client.asCurrentUser.indices.getDataLifecycle({
    name
  });
};
const getDataStreamsStats = (client, name = '*') => {
  return client.asCurrentUser.indices.dataStreamsStats({
    name,
    expand_wildcards: 'all',
    human: true
  });
};
const getMeteringStats = (client, name) => {
  let path = `/_metering/stats`;
  if (name) {
    path = `${path}/${name}`;
  }
  return client.asSecondaryAuthUser.transport.request({
    method: 'GET',
    path
  });
};
const getDataStreamsPrivileges = (client, names) => {
  return client.asCurrentUser.security.hasPrivileges({
    body: {
      index: [{
        names,
        privileges: ['delete_index', 'manage_data_stream_lifecycle']
      }]
    }
  });
};
function registerGetAllRoute({
  router,
  lib: {
    handleEsError
  },
  config
}) {
  const querySchema = _configSchema.schema.object({
    includeStats: _configSchema.schema.maybe(_configSchema.schema.oneOf([_configSchema.schema.literal('true'), _configSchema.schema.literal('false')]))
  });
  router.get({
    path: (0, _.addBasePath)('/data_streams'),
    validate: {
      query: querySchema
    }
  }, async (context, request, response) => {
    const {
      client
    } = (await context.core).elasticsearch;
    const includeStats = request.query.includeStats === 'true';
    try {
      const {
        data_streams: dataStreams
      } = await getDataStreams(client);
      let dataStreamsStats;
      let dataStreamsPrivileges;
      let meteringStats;
      if (includeStats && config.isDataStreamStatsEnabled !== false) {
        ({
          data_streams: dataStreamsStats
        } = await getDataStreamsStats(client));
      }
      if (includeStats && config.isSizeAndDocCountEnabled !== false) {
        ({
          datastreams: meteringStats
        } = await getMeteringStats(client));
      }
      if (config.isSecurityEnabled() && dataStreams.length > 0) {
        dataStreamsPrivileges = await getDataStreamsPrivileges(client, dataStreams.map(dataStream => dataStream.name));
      }
      const {
        index_templates: indexTemplates
      } = await client.asCurrentUser.indices.getIndexTemplate();
      const enhancedDataStreams = enhanceDataStreams({
        dataStreams,
        dataStreamsStats,
        meteringStats,
        dataStreamsPrivileges,
        indexTemplates
      });
      return response.ok({
        body: (0, _data_stream_serialization.deserializeDataStreamList)(enhancedDataStreams)
      });
    } catch (error) {
      return handleEsError({
        error,
        response
      });
    }
  });
}
function registerGetOneRoute({
  router,
  lib: {
    handleEsError
  },
  config
}) {
  const paramsSchema = _configSchema.schema.object({
    name: _configSchema.schema.string()
  });
  router.get({
    path: (0, _.addBasePath)('/data_streams/{name}'),
    validate: {
      params: paramsSchema
    }
  }, async (context, request, response) => {
    const {
      name
    } = request.params;
    const {
      client
    } = (await context.core).elasticsearch;
    let dataStreamsStats;
    let meteringStats;
    try {
      var _lifecycle$global_ret;
      const {
        data_streams: dataStreams
      } = await getDataStreams(client, name);
      const lifecycle = await getDataStreamLifecycle(client, name);
      // @ts-ignore - TS doesn't know about the `global_retention` property yet
      const globalMaxRetention = lifecycle === null || lifecycle === void 0 ? void 0 : (_lifecycle$global_ret = lifecycle.global_retention) === null || _lifecycle$global_ret === void 0 ? void 0 : _lifecycle$global_ret.max_retention;
      if (config.isDataStreamStatsEnabled !== false) {
        ({
          data_streams: dataStreamsStats
        } = await getDataStreamsStats(client, name));
      }
      if (config.isSizeAndDocCountEnabled !== false) {
        ({
          datastreams: meteringStats
        } = await getMeteringStats(client, name));
      }
      if (dataStreams[0]) {
        let dataStreamsPrivileges;
        let indexTemplates;
        if (config.isSecurityEnabled()) {
          dataStreamsPrivileges = await getDataStreamsPrivileges(client, [dataStreams[0].name]);
        }
        if (dataStreams[0].template) {
          const {
            index_templates: templates
          } = await client.asCurrentUser.indices.getIndexTemplate({
            name: dataStreams[0].template
          });
          if (templates) {
            indexTemplates = templates;
          }
        }
        const enhancedDataStreams = enhanceDataStreams({
          dataStreams,
          dataStreamsStats,
          meteringStats,
          dataStreamsPrivileges,
          globalMaxRetention,
          indexTemplates
        });
        const body = (0, _data_stream_serialization.deserializeDataStream)(enhancedDataStreams[0]);
        return response.ok({
          body
        });
      }
      return response.notFound();
    } catch (error) {
      return handleEsError({
        error,
        response
      });
    }
  });
}