"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.retrieveDocumentation = void 0;
var _tokens = require("../../utils/tokens");
var _summarize_document = require("./summarize_document");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const MAX_DOCUMENTS_DEFAULT = 3;
const MAX_TOKENS_DEFAULT = 1000;
const retrieveDocumentation = ({
  outputAPI,
  searchDocAPI,
  logger: log
}) => async ({
  searchTerm,
  connectorId,
  products,
  functionCalling,
  max = MAX_DOCUMENTS_DEFAULT,
  maxDocumentTokens = MAX_TOKENS_DEFAULT,
  tokenReductionStrategy = 'summarize'
}) => {
  try {
    const {
      results
    } = await searchDocAPI({
      query: searchTerm,
      products,
      max
    });
    log.debug(`searching with term=[${searchTerm}] returned ${results.length} documents`);
    const processedDocuments = await Promise.all(results.map(async document => {
      const tokenCount = (0, _tokens.count)(document.content);
      const docHasTooManyTokens = tokenCount >= maxDocumentTokens;
      log.debug(`processing doc [${document.url}] - tokens : [${tokenCount}] - tooManyTokens: [${docHasTooManyTokens}]`);
      let content = document.content;
      if (docHasTooManyTokens) {
        if (tokenReductionStrategy === 'summarize') {
          const extractResponse = await (0, _summarize_document.summarizeDocument)({
            searchTerm,
            documentContent: document.content,
            outputAPI,
            connectorId,
            functionCalling
          });
          content = (0, _tokens.truncate)(extractResponse.summary, maxDocumentTokens);
        } else {
          content = (0, _tokens.truncate)(document.content, maxDocumentTokens);
        }
      }
      log.debug(`done processing document [${document.url}]`);
      return {
        title: document.title,
        url: document.url,
        content
      };
    }));
    log.debug(() => {
      const docsAsJson = JSON.stringify(processedDocuments);
      return `searching with term=[${searchTerm}] - results: ${docsAsJson}`;
    });
    return {
      success: true,
      documents: processedDocuments.filter(doc => doc.content.length > 0)
    };
  } catch (e) {
    log.error(`Error retrieving documentation: ${e.message}. Returning empty results.`);
    return {
      success: false,
      documents: []
    };
  }
};
exports.retrieveDocumentation = retrieveDocumentation;