"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AI_ASSISTANT_KB_INFERENCE_ID = void 0;
exports.createInferenceEndpoint = createInferenceEndpoint;
exports.deleteInferenceEndpoint = deleteInferenceEndpoint;
exports.getElserModelStatus = getElserModelStatus;
exports.getInferenceEndpoint = getInferenceEndpoint;
exports.isInferenceEndpointMissingOrUnavailable = isInferenceEndpointMissingOrUnavailable;
var _elasticsearch = require("@elastic/elasticsearch");
var _moment = _interopRequireDefault(require("moment"));
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const AI_ASSISTANT_KB_INFERENCE_ID = exports.AI_ASSISTANT_KB_INFERENCE_ID = 'obs_ai_assistant_kb_inference';
async function createInferenceEndpoint({
  esClient,
  logger,
  modelId = '.elser_model_2'
}) {
  try {
    logger.debug(`Creating inference endpoint "${AI_ASSISTANT_KB_INFERENCE_ID}"`);
    return await esClient.asCurrentUser.inference.put({
      inference_id: AI_ASSISTANT_KB_INFERENCE_ID,
      task_type: 'sparse_embedding',
      inference_config: {
        service: 'elasticsearch',
        service_settings: {
          model_id: modelId,
          adaptive_allocations: {
            enabled: true,
            min_number_of_allocations: 1
          },
          num_threads: 1
        },
        task_settings: {}
      }
    }, {
      requestTimeout: _moment.default.duration(2, 'minutes').asMilliseconds()
    });
  } catch (e) {
    logger.debug(`Failed to create inference endpoint "${AI_ASSISTANT_KB_INFERENCE_ID}": ${e.message}`);
    throw e;
  }
}
async function deleteInferenceEndpoint({
  esClient
}) {
  const response = await esClient.asCurrentUser.inference.delete({
    inference_id: AI_ASSISTANT_KB_INFERENCE_ID,
    force: true
  });
  return response;
}
async function getInferenceEndpoint({
  esClient
}) {
  const response = await esClient.asInternalUser.inference.get({
    inference_id: AI_ASSISTANT_KB_INFERENCE_ID
  });
  if (response.endpoints.length > 0) {
    return response.endpoints[0];
  }
}
function isInferenceEndpointMissingOrUnavailable(error) {
  var _error$body, _error$body$error, _error$body2, _error$body2$error;
  return error instanceof _elasticsearch.errors.ResponseError && (((_error$body = error.body) === null || _error$body === void 0 ? void 0 : (_error$body$error = _error$body.error) === null || _error$body$error === void 0 ? void 0 : _error$body$error.type) === 'resource_not_found_exception' || ((_error$body2 = error.body) === null || _error$body2 === void 0 ? void 0 : (_error$body2$error = _error$body2.error) === null || _error$body2$error === void 0 ? void 0 : _error$body2$error.type) === 'status_exception');
}
async function getElserModelStatus({
  esClient,
  logger,
  config
}) {
  var _endpoint$service_set, _elserModelStats$depl, _elserModelStats$depl2, _elserModelStats$depl3, _elserModelStats$depl4;
  let errorMessage = '';
  const endpoint = await getInferenceEndpoint({
    esClient
  }).catch(error => {
    if (!isInferenceEndpointMissingOrUnavailable(error)) {
      throw error;
    }
    errorMessage = error.message;
  });
  const enabled = config.enableKnowledgeBase;
  if (!endpoint) {
    return {
      ready: false,
      enabled,
      errorMessage
    };
  }
  const modelId = (_endpoint$service_set = endpoint.service_settings) === null || _endpoint$service_set === void 0 ? void 0 : _endpoint$service_set.model_id;
  const modelStats = await esClient.asInternalUser.ml.getTrainedModelsStats({
    model_id: modelId
  }).catch(error => {
    logger.debug(`Failed to get model stats: ${error.message}`);
    errorMessage = error.message;
  });
  if (!modelStats) {
    return {
      ready: false,
      enabled,
      errorMessage
    };
  }
  const elserModelStats = modelStats.trained_model_stats.find(stats => {
    var _stats$deployment_sta;
    return ((_stats$deployment_sta = stats.deployment_stats) === null || _stats$deployment_sta === void 0 ? void 0 : _stats$deployment_sta.deployment_id) === AI_ASSISTANT_KB_INFERENCE_ID;
  });
  const deploymentState = elserModelStats === null || elserModelStats === void 0 ? void 0 : (_elserModelStats$depl = elserModelStats.deployment_stats) === null || _elserModelStats$depl === void 0 ? void 0 : _elserModelStats$depl.state;
  const allocationState = elserModelStats === null || elserModelStats === void 0 ? void 0 : (_elserModelStats$depl2 = elserModelStats.deployment_stats) === null || _elserModelStats$depl2 === void 0 ? void 0 : _elserModelStats$depl2.allocation_status.state;
  const allocationCount = (_elserModelStats$depl3 = elserModelStats === null || elserModelStats === void 0 ? void 0 : (_elserModelStats$depl4 = elserModelStats.deployment_stats) === null || _elserModelStats$depl4 === void 0 ? void 0 : _elserModelStats$depl4.allocation_status.allocation_count) !== null && _elserModelStats$depl3 !== void 0 ? _elserModelStats$depl3 : 0;
  const ready = deploymentState === 'started' && allocationState === 'fully_allocated' && allocationCount > 0;
  return {
    endpoint,
    ready,
    enabled,
    model_stats: {
      allocation_count: allocationCount,
      deployment_state: deploymentState,
      allocation_state: allocationState
    }
  };
}