"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.KnowledgeBaseService = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _boom = require("@hapi/boom");
var _lodash = require("lodash");
var _gptTokenizer = require("gpt-tokenizer");
var _ = require("..");
var _types = require("../../../common/types");
var _get_access_query = require("../util/get_access_query");
var _get_category_query = require("../util/get_category_query");
var _inference_endpoint = require("../inference_endpoint");
var _recall_from_search_connectors = require("./recall_from_search_connectors");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

function throwKnowledgeBaseNotReady(body) {
  throw (0, _boom.serverUnavailable)(`Knowledge base is not ready yet`, body);
}
class KnowledgeBaseService {
  constructor(dependencies) {
    (0, _defineProperty2.default)(this, "recall", async ({
      user,
      queries,
      categories,
      namespace,
      esClient,
      uiSettingsClient
    }) => {
      if (!this.dependencies.config.enableKnowledgeBase) {
        return [];
      }
      this.dependencies.logger.debug(() => `Recalling entries from KB for queries: "${JSON.stringify(queries)}"`);
      const [documentsFromKb, documentsFromConnectors] = await Promise.all([this.recallFromKnowledgeBase({
        user,
        queries,
        categories,
        namespace
      }).catch(error => {
        if ((0, _inference_endpoint.isInferenceEndpointMissingOrUnavailable)(error)) {
          throwKnowledgeBaseNotReady(error.body);
        }
        throw error;
      }), (0, _recall_from_search_connectors.recallFromSearchConnectors)({
        esClient,
        uiSettingsClient,
        queries,
        core: this.dependencies.core,
        logger: this.dependencies.logger
      }).catch(error => {
        this.dependencies.logger.debug('Error getting data from search indices');
        this.dependencies.logger.debug(error);
        return [];
      })]);
      this.dependencies.logger.debug(`documentsFromKb: ${JSON.stringify(documentsFromKb.slice(0, 5), null, 2)}`);
      this.dependencies.logger.debug(`documentsFromConnectors: ${JSON.stringify(documentsFromConnectors.slice(0, 5), null, 2)}`);
      const sortedEntries = (0, _lodash.orderBy)(documentsFromKb.concat(documentsFromConnectors), 'score', 'desc').slice(0, 20);
      const MAX_TOKENS = 4000;
      let tokenCount = 0;
      const returnedEntries = [];
      for (const entry of sortedEntries) {
        returnedEntries.push(entry);
        tokenCount += (0, _gptTokenizer.encode)(entry.text).length;
        if (tokenCount >= MAX_TOKENS) {
          break;
        }
      }
      const droppedEntries = sortedEntries.length - returnedEntries.length;
      if (droppedEntries > 0) {
        this.dependencies.logger.info(`Dropped ${droppedEntries} entries because of token limit`);
      }
      return returnedEntries;
    });
    (0, _defineProperty2.default)(this, "getUserInstructions", async (namespace, user) => {
      if (!this.dependencies.config.enableKnowledgeBase) {
        return [];
      }
      try {
        const response = await this.dependencies.esClient.asInternalUser.search({
          index: _.resourceNames.aliases.kb,
          query: {
            bool: {
              filter: [{
                term: {
                  type: _types.KnowledgeBaseType.UserInstruction
                }
              }, ...(0, _get_access_query.getAccessQuery)({
                user,
                namespace
              })]
            }
          },
          size: 500,
          _source: ['id', 'text', 'public']
        });
        return response.hits.hits.map(hit => {
          var _hit$_source$text, _hit$_source, _hit$_source2;
          return {
            id: hit._id,
            text: (_hit$_source$text = (_hit$_source = hit._source) === null || _hit$_source === void 0 ? void 0 : _hit$_source.text) !== null && _hit$_source$text !== void 0 ? _hit$_source$text : '',
            public: (_hit$_source2 = hit._source) === null || _hit$_source2 === void 0 ? void 0 : _hit$_source2.public
          };
        });
      } catch (error) {
        this.dependencies.logger.error('Failed to load instructions from knowledge base');
        this.dependencies.logger.error(error);
        return [];
      }
    });
    (0, _defineProperty2.default)(this, "getEntries", async ({
      query,
      sortBy,
      sortDirection
    }) => {
      if (!this.dependencies.config.enableKnowledgeBase) {
        return {
          entries: []
        };
      }
      try {
        const response = await this.dependencies.esClient.asInternalUser.search({
          index: _.resourceNames.aliases.kb,
          query: {
            bool: {
              filter: [
              // filter by search query
              ...(query ? [{
                query_string: {
                  query: `${query}*`,
                  fields: ['doc_id', 'title']
                }
              }] : []), {
                // exclude user instructions
                bool: {
                  must_not: {
                    term: {
                      type: _types.KnowledgeBaseType.UserInstruction
                    }
                  }
                }
              }]
            }
          },
          sort: sortBy === 'title' ? [{
            ['title.keyword']: {
              order: sortDirection
            }
          }, {
            doc_id: {
              order: sortDirection
            }
          } // sort by doc_id for backwards compatibility
          ] : [{
            [String(sortBy)]: {
              order: sortDirection
            }
          }],
          size: 500,
          _source: {
            includes: ['title', 'doc_id', 'text', 'is_correction', 'labels', 'confidence', 'public', '@timestamp', 'role', 'user.name', 'type']
          }
        });
        return {
          entries: response.hits.hits.map(hit => {
            var _title, _role;
            return {
              ...hit._source,
              title: (_title = hit._source.title) !== null && _title !== void 0 ? _title : hit._source.doc_id,
              // use `doc_id` as fallback title for backwards compatibility
              role: (_role = hit._source.role) !== null && _role !== void 0 ? _role : _types.KnowledgeBaseEntryRole.UserEntry,
              score: hit._score,
              id: hit._id
            };
          })
        };
      } catch (error) {
        if ((0, _inference_endpoint.isInferenceEndpointMissingOrUnavailable)(error)) {
          throwKnowledgeBaseNotReady(error.body);
        }
        throw error;
      }
    });
    (0, _defineProperty2.default)(this, "getPersonalUserInstructionId", async ({
      isPublic,
      user,
      namespace
    }) => {
      var _res$hits$hits$;
      if (!this.dependencies.config.enableKnowledgeBase) {
        return null;
      }
      const res = await this.dependencies.esClient.asInternalUser.search({
        index: _.resourceNames.aliases.kb,
        query: {
          bool: {
            filter: [{
              term: {
                type: _types.KnowledgeBaseType.UserInstruction
              }
            }, {
              term: {
                public: isPublic
              }
            }, ...(0, _get_access_query.getAccessQuery)({
              user,
              namespace
            })]
          }
        },
        size: 1,
        _source: false
      });
      return (_res$hits$hits$ = res.hits.hits[0]) === null || _res$hits$hits$ === void 0 ? void 0 : _res$hits$hits$._id;
    });
    (0, _defineProperty2.default)(this, "getUuidFromDocId", async ({
      docId,
      user,
      namespace
    }) => {
      var _response$hits$hits$;
      const query = {
        bool: {
          filter: [{
            term: {
              doc_id: docId
            }
          },
          // exclude user instructions
          {
            bool: {
              must_not: {
                term: {
                  type: _types.KnowledgeBaseType.UserInstruction
                }
              }
            }
          },
          // restrict access to user's own entries
          ...(0, _get_access_query.getAccessQuery)({
            user,
            namespace
          })]
        }
      };
      const response = await this.dependencies.esClient.asInternalUser.search({
        size: 1,
        index: _.resourceNames.aliases.kb,
        query,
        _source: false
      });
      return (_response$hits$hits$ = response.hits.hits[0]) === null || _response$hits$hits$ === void 0 ? void 0 : _response$hits$hits$._id;
    });
    (0, _defineProperty2.default)(this, "addEntry", async ({
      entry: {
        id,
        ...doc
      },
      user,
      namespace
    }) => {
      if (!this.dependencies.config.enableKnowledgeBase) {
        return;
      }
      try {
        await this.dependencies.esClient.asInternalUser.index({
          index: _.resourceNames.aliases.kb,
          id,
          document: {
            '@timestamp': new Date().toISOString(),
            ...doc,
            ...(doc.text ? {
              semantic_text: doc.text
            } : {}),
            user,
            namespace
          },
          refresh: 'wait_for'
        });
      } catch (error) {
        if ((0, _inference_endpoint.isInferenceEndpointMissingOrUnavailable)(error)) {
          throwKnowledgeBaseNotReady(error.body);
        }
        throw error;
      }
    });
    (0, _defineProperty2.default)(this, "deleteEntry", async ({
      id
    }) => {
      try {
        await this.dependencies.esClient.asInternalUser.delete({
          index: _.resourceNames.aliases.kb,
          id,
          refresh: 'wait_for'
        });
        return Promise.resolve();
      } catch (error) {
        if ((0, _inference_endpoint.isInferenceEndpointMissingOrUnavailable)(error)) {
          throwKnowledgeBaseNotReady(error.body);
        }
        throw error;
      }
    });
    (0, _defineProperty2.default)(this, "getStatus", async () => {
      return (0, _inference_endpoint.getElserModelStatus)({
        esClient: this.dependencies.esClient,
        logger: this.dependencies.logger,
        config: this.dependencies.config
      });
    });
    this.dependencies = dependencies;
  }
  async setup(esClient, modelId) {
    await (0, _inference_endpoint.deleteInferenceEndpoint)({
      esClient
    }).catch(e => {}); // ensure existing inference endpoint is deleted
    return (0, _inference_endpoint.createInferenceEndpoint)({
      esClient,
      logger: this.dependencies.logger,
      modelId
    });
  }
  async reset(esClient) {
    try {
      await (0, _inference_endpoint.deleteInferenceEndpoint)({
        esClient
      });
    } catch (error) {
      if ((0, _inference_endpoint.isInferenceEndpointMissingOrUnavailable)(error)) {
        return;
      }
      throw error;
    }
  }
  async recallFromKnowledgeBase({
    queries,
    categories,
    namespace,
    user
  }) {
    const response = await this.dependencies.esClient.asInternalUser.search({
      index: [_.resourceNames.aliases.kb],
      query: {
        bool: {
          should: queries.map(({
            text,
            boost = 1
          }) => ({
            semantic: {
              field: 'semantic_text',
              query: text,
              boost
            }
          })),
          filter: [...(0, _get_access_query.getAccessQuery)({
            user,
            namespace
          }), ...(0, _get_category_query.getCategoryQuery)({
            categories
          }),
          // exclude user instructions
          {
            bool: {
              must_not: {
                term: {
                  type: _types.KnowledgeBaseType.UserInstruction
                }
              }
            }
          }]
        }
      },
      size: 20,
      _source: {
        includes: ['text', 'is_correction', 'labels', 'doc_id', 'title']
      }
    });
    return response.hits.hits.map(hit => {
      var _hit$_source3, _hit$_source4, _hit$_source5, _hit$_source$title, _hit$_source6, _hit$_source7;
      return {
        text: (_hit$_source3 = hit._source) === null || _hit$_source3 === void 0 ? void 0 : _hit$_source3.text,
        is_correction: (_hit$_source4 = hit._source) === null || _hit$_source4 === void 0 ? void 0 : _hit$_source4.is_correction,
        labels: (_hit$_source5 = hit._source) === null || _hit$_source5 === void 0 ? void 0 : _hit$_source5.labels,
        title: (_hit$_source$title = (_hit$_source6 = hit._source) === null || _hit$_source6 === void 0 ? void 0 : _hit$_source6.title) !== null && _hit$_source$title !== void 0 ? _hit$_source$title : (_hit$_source7 = hit._source) === null || _hit$_source7 === void 0 ? void 0 : _hit$_source7.doc_id,
        // use `doc_id` as fallback title for backwards compatibility
        score: hit._score,
        id: hit._id
      };
    });
  }
}
exports.KnowledgeBaseService = KnowledgeBaseService;