"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setupConversationAndKbIndexAssets = setupConversationAndKbIndexAssets;
var _server = require("@kbn/alerting-plugin/server");
var _conversation_component_template = require("./conversation_component_template");
var _kb_component_template = require("./kb_component_template");
var _ = require(".");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

async function setupConversationAndKbIndexAssets({
  logger,
  core
}) {
  try {
    logger.debug('Setting up index assets');
    const [coreStart] = await core.getStartServices();
    const {
      asInternalUser
    } = coreStart.elasticsearch.client;

    // Conversations: component template
    await asInternalUser.cluster.putComponentTemplate({
      create: false,
      name: _.resourceNames.componentTemplate.conversations,
      template: _conversation_component_template.conversationComponentTemplate
    });

    // Conversations: index template
    await asInternalUser.indices.putIndexTemplate({
      name: _.resourceNames.indexTemplate.conversations,
      composed_of: [_.resourceNames.componentTemplate.conversations],
      create: false,
      index_patterns: [_.resourceNames.indexPatterns.conversations],
      template: {
        settings: {
          number_of_shards: 1,
          auto_expand_replicas: '0-1',
          hidden: true
        }
      }
    });

    // Conversations: write index
    const conversationAliasName = _.resourceNames.aliases.conversations;
    await (0, _server.createConcreteWriteIndex)({
      esClient: asInternalUser,
      logger,
      totalFieldsLimit: 10000,
      indexPatterns: {
        alias: conversationAliasName,
        pattern: `${conversationAliasName}*`,
        basePattern: `${conversationAliasName}*`,
        name: `${conversationAliasName}-000001`,
        template: _.resourceNames.indexTemplate.conversations
      },
      dataStreamAdapter: (0, _server.getDataStreamAdapter)({
        useDataStreamForAlerts: false
      })
    });

    // Knowledge base: component template
    await asInternalUser.cluster.putComponentTemplate({
      create: false,
      name: _.resourceNames.componentTemplate.kb,
      template: _kb_component_template.kbComponentTemplate
    });

    // Knowledge base: index template
    await asInternalUser.indices.putIndexTemplate({
      name: _.resourceNames.indexTemplate.kb,
      composed_of: [_.resourceNames.componentTemplate.kb],
      create: false,
      index_patterns: [_.resourceNames.indexPatterns.kb],
      template: {
        settings: {
          number_of_shards: 1,
          auto_expand_replicas: '0-1',
          hidden: true
        }
      }
    });

    // Knowledge base: write index
    const kbAliasName = _.resourceNames.aliases.kb;
    await (0, _server.createConcreteWriteIndex)({
      esClient: asInternalUser,
      logger,
      totalFieldsLimit: 10000,
      indexPatterns: {
        alias: kbAliasName,
        pattern: `${kbAliasName}*`,
        basePattern: `${kbAliasName}*`,
        name: `${kbAliasName}-000001`,
        template: _.resourceNames.indexTemplate.kb
      },
      dataStreamAdapter: (0, _server.getDataStreamAdapter)({
        useDataStreamForAlerts: false
      })
    });
    logger.info('Successfully set up index assets');
  } catch (error) {
    logger.error(`Failed setting up index assets: ${error.message}`);
    logger.debug(error);
  }
}