"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.registerMigrateKnowledgeBaseEntriesTask = registerMigrateKnowledgeBaseEntriesTask;
exports.runSemanticTextKnowledgeBaseMigration = runSemanticTextKnowledgeBaseMigration;
exports.scheduleSemanticTextMigration = scheduleSemanticTextMigration;
var _pLimit = _interopRequireDefault(require("p-limit"));
var _pRetry = _interopRequireDefault(require("p-retry"));
var _ = require("..");
var _inference_endpoint = require("../inference_endpoint");
var _setup_conversation_and_kb_index_assets = require("../setup_conversation_and_kb_index_assets");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const TASK_ID = 'obs-ai-assistant:knowledge-base-migration-task-id';
const TASK_TYPE = 'obs-ai-assistant:knowledge-base-migration';

// This task will re-index all knowledge base entries without `semantic_text` field
// to ensure the field is populated with the correct embeddings.
// After the migration we will no longer need to use the `ml.tokens` field.
async function registerMigrateKnowledgeBaseEntriesTask({
  taskManager,
  logger,
  core,
  config
}) {
  const [coreStart, pluginsStart] = await core.getStartServices();
  try {
    logger.debug(`Register task "${TASK_TYPE}"`);
    taskManager.registerTaskDefinitions({
      [TASK_TYPE]: {
        title: 'Migrate AI Assistant Knowledge Base',
        description: `Migrates AI Assistant knowledge base entries`,
        timeout: '1h',
        maxAttempts: 5,
        createTaskRunner() {
          return {
            async run() {
              logger.debug(`Run task: "${TASK_TYPE}"`);
              const esClient = coreStart.elasticsearch.client;
              const hasKbIndex = await esClient.asInternalUser.indices.exists({
                index: _.resourceNames.aliases.kb
              });
              if (!hasKbIndex) {
                logger.debug('Knowledge base index does not exist. Skipping semantic text migration.');
                return;
              }

              // update fields and mappings
              await (0, _setup_conversation_and_kb_index_assets.setupConversationAndKbIndexAssets)({
                logger,
                core
              });

              // run migration
              await runSemanticTextKnowledgeBaseMigration({
                esClient,
                logger,
                config
              });
            }
          };
        }
      }
    });
  } catch (error) {
    logger.error(`Failed to register task "${TASK_TYPE}". Error: ${error}`);
  }
  try {
    logger.debug(`Scheduled task: "${TASK_TYPE}"`);
    await scheduleSemanticTextMigration(pluginsStart);
  } catch (error) {
    logger.error(`Failed to schedule task "${TASK_TYPE}". Error: ${error}`);
  }
}
function scheduleSemanticTextMigration(pluginsStart) {
  return pluginsStart.taskManager.ensureScheduled({
    id: TASK_ID,
    taskType: TASK_TYPE,
    scope: ['aiAssistant'],
    params: {},
    state: {}
  });
}
async function runSemanticTextKnowledgeBaseMigration({
  esClient,
  logger,
  config
}) {
  logger.debug('Knowledge base migration: Running migration');
  try {
    const response = await esClient.asInternalUser.search({
      size: 100,
      track_total_hits: true,
      index: [_.resourceNames.aliases.kb],
      query: {
        bool: {
          must_not: {
            exists: {
              field: 'semantic_text'
            }
          }
        }
      },
      _source: {
        excludes: ['ml.tokens']
      }
    });
    if (response.hits.hits.length === 0) {
      logger.debug('Knowledge base migration: No remaining entries to migrate');
      return;
    }
    logger.debug(`Knowledge base migration: Found ${response.hits.hits.length} entries to migrate`);
    await waitForModel({
      esClient,
      logger,
      config
    });

    // Limit the number of concurrent requests to avoid overloading the cluster
    const limiter = (0, _pLimit.default)(10);
    const promises = response.hits.hits.map(hit => {
      return limiter(() => {
        if (!hit._source || !hit._id) {
          return;
        }
        return esClient.asInternalUser.update({
          refresh: 'wait_for',
          index: _.resourceNames.aliases.kb,
          id: hit._id,
          body: {
            doc: {
              ...hit._source,
              semantic_text: hit._source.text
            }
          }
        });
      });
    });
    await Promise.all(promises);
    logger.debug(`Knowledge base migration: Migrated ${promises.length} entries`);
    await runSemanticTextKnowledgeBaseMigration({
      esClient,
      logger,
      config
    });
  } catch (e) {
    logger.error(`Knowledge base migration failed: ${e.message}`);
  }
}
async function waitForModel({
  esClient,
  logger,
  config
}) {
  return (0, _pRetry.default)(async () => {
    const {
      ready
    } = await (0, _inference_endpoint.getElserModelStatus)({
      esClient,
      logger,
      config
    });
    if (!ready) {
      logger.debug('Elser model is not yet ready. Retrying...');
      throw new Error('Elser model is not yet ready');
    }
  }, {
    retries: 30,
    factor: 2,
    maxTimeout: 30_000
  });
}