"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createObservabilityEsClient = createObservabilityEsClient;
var _apmUtils = require("@kbn/apm-utils");
var _esql_result_to_plain_objects = require("../esql_result_to_plain_objects");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

/**
 * An Elasticsearch Client with a fully typed `search` method and built-in
 * APM instrumentation.
 */

function createObservabilityEsClient({
  client,
  logger,
  plugin
}) {
  // wraps the ES calls in a named APM span for better analysis
  // (otherwise it would just eg be a _search span)
  const callWithLogger = (operationName, request, callback) => {
    logger.debug(() => `Request (${operationName}):\n${JSON.stringify(request)}`);
    return (0, _apmUtils.withSpan)({
      name: operationName,
      labels: {
        plugin
      }
    }, callback, logger).then(response => {
      logger.trace(() => `Response (${operationName}):\n${JSON.stringify(response, null, 2)}`);
      return response;
    });
  };
  return {
    client,
    fieldCaps(operationName, parameters) {
      return callWithLogger(operationName, parameters, () => {
        return client.fieldCaps({
          ...parameters
        });
      });
    },
    esql(operationName, {
      parseOutput = true,
      format = 'json',
      columnar = false,
      ...parameters
    }) {
      logger.trace(() => `Request (${operationName}):\n${JSON.stringify(parameters, null, 2)}`);
      return (0, _apmUtils.withSpan)({
        name: operationName,
        labels: {
          plugin
        }
      }, () => {
        return client.esql.query({
          ...parameters,
          format,
          columnar
        }, {
          querystring: {
            drop_null_columns: true
          }
        });
      }).then(response => {
        logger.trace(() => `Response (${operationName}):\n${JSON.stringify(response, null, 2)}`);
        const esqlResponse = response;
        const shouldParseOutput = parseOutput && !columnar && format === 'json';
        return shouldParseOutput ? (0, _esql_result_to_plain_objects.esqlResultToPlainObjects)(esqlResponse) : esqlResponse;
      }).catch(error => {
        throw error;
      });
    },
    search(operationName, parameters) {
      return callWithLogger(operationName, parameters, () => {
        return client.search(parameters);
      });
    },
    msearch(operationName, parameters) {
      return callWithLogger(operationName, parameters, () => {
        return client.msearch(parameters);
      });
    }
  };
}