"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DocumentationManager = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _server = require("@kbn/task-manager-plugin/server");
var _tasks = require("../../tasks");
var _check_license = require("./check_license");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const TEN_MIN_IN_MS = 10 * 60 * 1000;

/**
 * High-level installation service, handling product documentation
 * installation as unary operations, abstracting away the fact
 * that documentation is composed of multiple entities.
 */
class DocumentationManager {
  constructor({
    logger,
    taskManager,
    licensing,
    docInstallClient,
    auditService
  }) {
    (0, _defineProperty2.default)(this, "logger", void 0);
    (0, _defineProperty2.default)(this, "taskManager", void 0);
    (0, _defineProperty2.default)(this, "licensing", void 0);
    (0, _defineProperty2.default)(this, "docInstallClient", void 0);
    (0, _defineProperty2.default)(this, "auditService", void 0);
    this.logger = logger;
    this.taskManager = taskManager;
    this.licensing = licensing;
    this.docInstallClient = docInstallClient;
    this.auditService = auditService;
  }
  async install(options = {}) {
    const {
      request,
      force = false,
      wait = false
    } = options;
    const {
      status
    } = await this.getStatus();
    if (!force && status === 'installed') {
      return;
    }
    const license = await this.licensing.getLicense();
    if (!(0, _check_license.checkLicense)(license)) {
      throw new Error('Elastic documentation requires an enterprise license');
    }
    const taskId = await (0, _tasks.scheduleInstallAllTask)({
      taskManager: this.taskManager,
      logger: this.logger
    });
    if (request) {
      this.auditService.asScoped(request).log({
        message: `User is requesting installation of product documentation for AI Assistants. Task ID=[${taskId}]`,
        event: {
          action: 'product_documentation_create',
          category: ['database'],
          type: ['creation'],
          outcome: 'unknown'
        }
      });
    }
    if (wait) {
      await (0, _tasks.waitUntilTaskCompleted)({
        taskManager: this.taskManager,
        taskId,
        timeout: TEN_MIN_IN_MS
      });
    }
  }
  async update(options = {}) {
    const {
      request,
      wait = false
    } = options;
    const taskId = await (0, _tasks.scheduleEnsureUpToDateTask)({
      taskManager: this.taskManager,
      logger: this.logger
    });
    if (request) {
      this.auditService.asScoped(request).log({
        message: `User is requesting update of product documentation for AI Assistants. Task ID=[${taskId}]`,
        event: {
          action: 'product_documentation_update',
          category: ['database'],
          type: ['change'],
          outcome: 'unknown'
        }
      });
    }
    if (wait) {
      await (0, _tasks.waitUntilTaskCompleted)({
        taskManager: this.taskManager,
        taskId,
        timeout: TEN_MIN_IN_MS
      });
    }
  }
  async uninstall(options = {}) {
    const {
      request,
      wait = false
    } = options;
    const taskId = await (0, _tasks.scheduleUninstallAllTask)({
      taskManager: this.taskManager,
      logger: this.logger
    });
    if (request) {
      this.auditService.asScoped(request).log({
        message: `User is requesting deletion of product documentation for AI Assistants. Task ID=[${taskId}]`,
        event: {
          action: 'product_documentation_delete',
          category: ['database'],
          type: ['deletion'],
          outcome: 'unknown'
        }
      });
    }
    if (wait) {
      await (0, _tasks.waitUntilTaskCompleted)({
        taskManager: this.taskManager,
        taskId,
        timeout: TEN_MIN_IN_MS
      });
    }
  }
  async getStatus() {
    const taskStatus = await (0, _tasks.getTaskStatus)({
      taskManager: this.taskManager,
      taskId: _tasks.INSTALL_ALL_TASK_ID
    });
    if (taskStatus !== 'not_scheduled') {
      const status = convertTaskStatus(taskStatus);
      if (status !== 'unknown') {
        return {
          status
        };
      }
    }
    const installStatus = await this.docInstallClient.getInstallationStatus();
    const overallStatus = getOverallStatus(Object.values(installStatus).map(v => v.status));
    return {
      status: overallStatus
    };
  }
}
exports.DocumentationManager = DocumentationManager;
const convertTaskStatus = taskStatus => {
  switch (taskStatus) {
    case _server.TaskStatus.Idle:
    case _server.TaskStatus.Claiming:
    case _server.TaskStatus.Running:
      return 'installing';
    case _server.TaskStatus.Failed:
      return 'error';
    case _server.TaskStatus.Unrecognized:
    case _server.TaskStatus.DeadLetter:
    case _server.TaskStatus.ShouldDelete:
    default:
      return 'unknown';
  }
};
const getOverallStatus = statuses => {
  const statusOrder = ['error', 'installing', 'uninstalled', 'installed'];
  for (const status of statusOrder) {
    if (statuses.includes(status)) {
      return status;
    }
  }
  return 'installed';
};