"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PackageInstaller = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _productDocCommon = require("@kbn/product-doc-common");
var _utils = require("./utils");
var _semver = require("./utils/semver");
var _steps = require("./steps");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

class PackageInstaller {
  constructor({
    artifactsFolder,
    logger,
    esClient,
    productDocClient,
    endpointManager,
    artifactRepositoryUrl,
    kibanaVersion
  }) {
    (0, _defineProperty2.default)(this, "log", void 0);
    (0, _defineProperty2.default)(this, "artifactsFolder", void 0);
    (0, _defineProperty2.default)(this, "esClient", void 0);
    (0, _defineProperty2.default)(this, "productDocClient", void 0);
    (0, _defineProperty2.default)(this, "endpointManager", void 0);
    (0, _defineProperty2.default)(this, "artifactRepositoryUrl", void 0);
    (0, _defineProperty2.default)(this, "currentVersion", void 0);
    this.esClient = esClient;
    this.productDocClient = productDocClient;
    this.artifactsFolder = artifactsFolder;
    this.endpointManager = endpointManager;
    this.artifactRepositoryUrl = artifactRepositoryUrl;
    this.currentVersion = (0, _semver.majorMinor)(kibanaVersion);
    this.log = logger;
  }

  /**
   * Make sure that the currently installed doc packages are up to date.
   * Will not upgrade products that are not already installed
   */
  async ensureUpToDate({}) {
    const [repositoryVersions, installStatuses] = await Promise.all([(0, _steps.fetchArtifactVersions)({
      artifactRepositoryUrl: this.artifactRepositoryUrl
    }), this.productDocClient.getInstallationStatus()]);
    const toUpdate = [];
    Object.entries(installStatuses).forEach(([productName, productState]) => {
      if (productState.status === 'uninstalled') {
        return;
      }
      const availableVersions = repositoryVersions[productName];
      if (!availableVersions || !availableVersions.length) {
        return;
      }
      const selectedVersion = selectVersion(this.currentVersion, availableVersions);
      if (productState.version !== selectedVersion) {
        toUpdate.push({
          productName: productName,
          productVersion: selectedVersion
        });
      }
    });
    for (const {
      productName,
      productVersion
    } of toUpdate) {
      await this.installPackage({
        productName,
        productVersion
      });
    }
  }
  async installAll({}) {
    const repositoryVersions = await (0, _steps.fetchArtifactVersions)({
      artifactRepositoryUrl: this.artifactRepositoryUrl
    });
    const allProducts = Object.values(_productDocCommon.DocumentationProduct);
    for (const productName of allProducts) {
      const availableVersions = repositoryVersions[productName];
      if (!availableVersions || !availableVersions.length) {
        this.log.warn(`No version found for product [${productName}]`);
        continue;
      }
      const selectedVersion = selectVersion(this.currentVersion, availableVersions);
      await this.installPackage({
        productName,
        productVersion: selectedVersion
      });
    }
  }
  async installPackage({
    productName,
    productVersion
  }) {
    this.log.info(`Starting installing documentation for product [${productName}] and version [${productVersion}]`);
    productVersion = (0, _semver.majorMinor)(productVersion);
    await this.uninstallPackage({
      productName
    });
    let zipArchive;
    try {
      await this.productDocClient.setInstallationStarted({
        productName,
        productVersion
      });
      await this.endpointManager.ensureInternalElserInstalled();
      const artifactFileName = (0, _productDocCommon.getArtifactName)({
        productName,
        productVersion
      });
      const artifactUrl = `${this.artifactRepositoryUrl}/${artifactFileName}`;
      const artifactPath = `${this.artifactsFolder}/${artifactFileName}`;
      this.log.debug(`Downloading from [${artifactUrl}] to [${artifactPath}]`);
      await (0, _utils.downloadToDisk)(artifactUrl, artifactPath);
      zipArchive = await (0, _utils.openZipArchive)(artifactPath);
      (0, _steps.validateArtifactArchive)(zipArchive);
      const mappings = await (0, _utils.loadMappingFile)(zipArchive);
      const indexName = (0, _productDocCommon.getProductDocIndexName)(productName);
      await (0, _steps.createIndex)({
        indexName,
        mappings,
        esClient: this.esClient,
        log: this.log
      });
      await (0, _steps.populateIndex)({
        indexName,
        archive: zipArchive,
        esClient: this.esClient,
        log: this.log
      });
      await this.productDocClient.setInstallationSuccessful(productName, indexName);
      this.log.info(`Documentation installation successful for product [${productName}] and version [${productVersion}]`);
    } catch (e) {
      this.log.error(`Error during documentation installation of product [${productName}]/[${productVersion}] : ${e.message}`);
      await this.productDocClient.setInstallationFailed(productName, e.message);
      throw e;
    } finally {
      var _zipArchive;
      (_zipArchive = zipArchive) === null || _zipArchive === void 0 ? void 0 : _zipArchive.close();
    }
  }
  async uninstallPackage({
    productName
  }) {
    const indexName = (0, _productDocCommon.getProductDocIndexName)(productName);
    await this.esClient.indices.delete({
      index: indexName
    }, {
      ignore: [404]
    });
    await this.productDocClient.setUninstalled(productName);
  }
  async uninstallAll() {
    const allProducts = Object.values(_productDocCommon.DocumentationProduct);
    for (const productName of allProducts) {
      await this.uninstallPackage({
        productName
      });
    }
  }
}
exports.PackageInstaller = PackageInstaller;
const selectVersion = (currentVersion, availableVersions) => {
  return availableVersions.includes(currentVersion) ? currentVersion : (0, _semver.latestVersion)(availableVersions);
};