"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getReplacedByForPrivilege = getReplacedByForPrivilege;
exports.privilegesFactory = privilegesFactory;
var _lodash = require("lodash");
var _securityAuthorizationCoreCommon = require("@kbn/security-authorization-core-common");
var _securityPluginTypesServer = require("@kbn/security-plugin-types-server");
var _feature_privilege_builder = require("./feature_privilege_builder");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

function privilegesFactory(actions, featuresService, licenseService) {
  const featurePrivilegeBuilder = (0, _feature_privilege_builder.featurePrivilegeBuilderFactory)(actions);
  return {
    get(respectLicenseLevel = true) {
      const features = featuresService.getKibanaFeatures();
      const {
        allowSubFeaturePrivileges
      } = licenseService.getFeatures();
      const {
        hasAtLeast: licenseHasAtLeast
      } = licenseService;
      const basePrivilegeFeatures = features.filter(feature => !feature.excludeFromBasePrivileges);
      const allActionsSet = new Set();
      const readActionsSet = new Set();
      basePrivilegeFeatures.forEach(feature => {
        if (feature.hidden) {
          return;
        }
        for (const {
          privilegeId,
          privilege
        } of featuresService.featurePrivilegeIterator(feature, {
          augmentWithSubFeaturePrivileges: true,
          licenseHasAtLeast,
          predicate: (pId, featurePrivilege) => !featurePrivilege.excludeFromBasePrivileges
        })) {
          const privilegeActions = featurePrivilegeBuilder.getActions(privilege, feature);
          privilegeActions.forEach(action => {
            allActionsSet.add(action);
            if (privilegeId === 'read') {
              readActionsSet.add(action);
            }
          });
        }
      });

      // Remember privilege as composable to update it later, once actions for all referenced privileges are also
      // calculated and registered.
      const composablePrivileges = [];
      const tryStoreComposablePrivilege = (feature, privilegeId, privilege) => {
        // If privilege is configured with `composedOf` it should be complemented with **all**
        // actions from referenced privileges.
        if (privilege.composedOf) {
          composablePrivileges.push({
            featureId: feature.id,
            privilegeId,
            references: privilege.composedOf,
            excludeFromBasePrivileges: feature.excludeFromBasePrivileges || privilege.excludeFromBasePrivileges
          });
        }

        // If a privilege is configured with `replacedBy`, it's part of the deprecated feature and
        // should be complemented with the subset of actions from the referenced privileges to
        // maintain backward compatibility. Namely, deprecated privileges should grant the same UI
        // capabilities and alerting actions as the privileges that replace them, so that the
        // client-side code can safely use only non-deprecated UI capabilities and users can still
        // access previously created alerting rules and alerts.
        const replacedBy = getReplacedByForPrivilege(privilegeId, privilege);
        if (replacedBy) {
          composablePrivileges.push({
            featureId: feature.id,
            privilegeId,
            references: replacedBy,
            actionsFilter: action => actions.ui.isValid(action) || actions.alerting.isValid(action)
          });
        }
      };
      const hiddenFeatures = new Set();
      const featurePrivileges = {};
      for (const feature of features) {
        var _feature$subFeatures;
        featurePrivileges[feature.id] = {};
        for (const featurePrivilege of featuresService.featurePrivilegeIterator(feature, {
          augmentWithSubFeaturePrivileges: true,
          licenseHasAtLeast
        })) {
          const fullPrivilegeId = featurePrivilege.privilegeId;
          featurePrivileges[feature.id][fullPrivilegeId] = [actions.login, ...(0, _lodash.uniq)(featurePrivilegeBuilder.getActions(featurePrivilege.privilege, feature))];
          tryStoreComposablePrivilege(feature, fullPrivilegeId, featurePrivilege.privilege);
        }
        for (const featurePrivilege of featuresService.featurePrivilegeIterator(feature, {
          augmentWithSubFeaturePrivileges: false,
          licenseHasAtLeast
        })) {
          const minimalPrivilegeId = (0, _securityAuthorizationCoreCommon.getMinimalPrivilegeId)(featurePrivilege.privilegeId);
          featurePrivileges[feature.id][minimalPrivilegeId] = [actions.login, ...(0, _lodash.uniq)(featurePrivilegeBuilder.getActions(featurePrivilege.privilege, feature))];
          tryStoreComposablePrivilege(feature, minimalPrivilegeId, featurePrivilege.privilege);
        }
        if ((!respectLicenseLevel || allowSubFeaturePrivileges) && ((_feature$subFeatures = feature.subFeatures) === null || _feature$subFeatures === void 0 ? void 0 : _feature$subFeatures.length) > 0) {
          for (const subFeaturePrivilege of featuresService.subFeaturePrivilegeIterator(feature, licenseHasAtLeast)) {
            featurePrivileges[feature.id][subFeaturePrivilege.id] = [actions.login, ...(0, _lodash.uniq)(featurePrivilegeBuilder.getActions(subFeaturePrivilege, feature))];
            tryStoreComposablePrivilege(feature, subFeaturePrivilege.id, subFeaturePrivilege);
          }
        }
        if (feature.hidden || Object.keys(featurePrivileges[feature.id]).length === 0) {
          hiddenFeatures.add(feature.id);
        }
      }

      // Update composable feature privileges to include and deduplicate actions from the referenced privileges.
      // Note that we should do it _before_ removing hidden features. Also, currently, feature privilege composition
      // doesn't respect the minimum license level required by the feature whose privileges are being included in
      // another feature. This could potentially enable functionality in a license lower than originally intended. It
      // might or might not be desired, but we're accepting this for now, as every attempt to compose a feature
      // undergoes a stringent review process.
      for (const composableFeature of composablePrivileges) {
        const composedActions = composableFeature.references.flatMap(privilegeReference => privilegeReference.privileges.flatMap(privilege => {
          var _featurePrivileges$pr;
          const privilegeActions = (_featurePrivileges$pr = featurePrivileges[privilegeReference.feature][privilege]) !== null && _featurePrivileges$pr !== void 0 ? _featurePrivileges$pr : [];
          return composableFeature.actionsFilter ? privilegeActions.filter(composableFeature.actionsFilter) : privilegeActions;
        }));
        featurePrivileges[composableFeature.featureId][composableFeature.privilegeId] = [...new Set(featurePrivileges[composableFeature.featureId][composableFeature.privilegeId].concat(composedActions))];
        if (!composableFeature.excludeFromBasePrivileges) {
          for (const action of composedActions) {
            // Login action is special since it's added explicitly for feature and base privileges.
            if (action === actions.login) {
              continue;
            }
            allActionsSet.add(action);
            if (composableFeature.privilegeId === 'read') {
              readActionsSet.add(action);
            }
          }
        }
      }

      // Remove hidden features to avoid registering standalone privileges for them.
      for (const hiddenFeatureId of hiddenFeatures) {
        delete featurePrivileges[hiddenFeatureId];
      }
      const allActions = [...allActionsSet];
      const readActions = [...readActionsSet];
      return {
        features: featurePrivileges,
        global: {
          all: [actions.login, actions.api.get(_securityPluginTypesServer.ApiOperation.Read, 'decryptedTelemetry'), actions.api.get(_securityPluginTypesServer.ApiOperation.Read, 'features'), actions.api.get(_securityPluginTypesServer.ApiOperation.Manage, 'taskManager'), actions.api.get(_securityPluginTypesServer.ApiOperation.Manage, 'spaces'), actions.space.manage, actions.ui.get('spaces', 'manage'), actions.ui.get('management', 'kibana', 'spaces'), actions.ui.get('catalogue', 'spaces'), actions.ui.get('enterpriseSearch', 'all'), actions.ui.get('globalSettings', 'save'), actions.ui.get('globalSettings', 'show'), ...allActions],
          read: [actions.login, actions.api.get(_securityPluginTypesServer.ApiOperation.Read, 'decryptedTelemetry'), actions.ui.get('globalSettings', 'show'), ...readActions]
        },
        space: {
          all: [actions.login, ...allActions],
          read: [actions.login, ...readActions]
        },
        reserved: features.reduce((acc, feature) => {
          if (feature.reserved) {
            feature.reserved.privileges.forEach(reservedPrivilege => {
              acc[reservedPrivilege.id] = [...(0, _lodash.uniq)(featurePrivilegeBuilder.getActions(reservedPrivilege.privilege, feature))];
            });
          }
          return acc;
        }, {})
      };
    }
  };
}

/**
 * Returns a list of privileges that replace the given privilege, if any. Works for both top-level
 * and sub-feature privileges.
 * @param privilegeId The ID of the privilege to get replacements for.
 * @param privilege The privilege definition to get replacements for.
 */
function getReplacedByForPrivilege(privilegeId, privilege) {
  const replacedBy = privilege.replacedBy;
  if (!replacedBy) {
    return;
  }

  // If a privilege of the deprecated feature explicitly defines a replacement for minimal privileges, use it.
  // Otherwise, use the default replacement for all cases.
  return 'minimal' in replacedBy ? (0, _securityAuthorizationCoreCommon.isMinimalPrivilegeId)(privilegeId) ? replacedBy.minimal : replacedBy.default : replacedBy;
}