"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initAPIAuthorization = initAPIAuthorization;
var _server = require("@kbn/core/server");
var _constants = require("../../common/constants");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const isAuthzDisabled = authz => {
  return (authz === null || authz === void 0 ? void 0 : authz.enabled) === false;
};
const isReservedPrivilegeSet = privilege => {
  return Object.hasOwn(_server.ReservedPrivilegesSet, privilege);
};
function initAPIAuthorization(http, {
  actions,
  checkPrivilegesDynamicallyWithRequest,
  checkPrivilegesWithRequest,
  mode
}, logger) {
  http.registerOnPostAuth(async (request, response, toolkit) => {
    // if we aren't using RBAC for this request, just continue
    if (!mode.useRbacForRequest(request)) {
      return toolkit.next();
    }
    const security = request.route.options.security;
    if (security) {
      if (isAuthzDisabled(security.authz)) {
        return toolkit.next();
      }
      const authz = security.authz;
      const {
        requestedPrivileges,
        requestedReservedPrivileges
      } = authz.requiredPrivileges.reduce((acc, privilegeEntry) => {
        var _privilegeEntry$allRe, _privilegeEntry$anyRe;
        const privileges = typeof privilegeEntry === 'object' ? [...((_privilegeEntry$allRe = privilegeEntry.allRequired) !== null && _privilegeEntry$allRe !== void 0 ? _privilegeEntry$allRe : []), ...((_privilegeEntry$anyRe = privilegeEntry.anyRequired) !== null && _privilegeEntry$anyRe !== void 0 ? _privilegeEntry$anyRe : [])] : [privilegeEntry];
        for (const privilege of privileges) {
          if (isReservedPrivilegeSet(privilege)) {
            acc.requestedReservedPrivileges.push(privilege);
          } else {
            acc.requestedPrivileges.push(privilege);
          }
        }
        return acc;
      }, {
        requestedPrivileges: [],
        requestedReservedPrivileges: []
      });
      const checkPrivileges = checkPrivilegesDynamicallyWithRequest(request);
      const privilegeToApiOperation = privilege => privilege.replace(_constants.API_OPERATION_PREFIX, '');
      const kibanaPrivileges = {};
      if (requestedPrivileges.length > 0) {
        const checkPrivilegesResponse = await checkPrivileges({
          kibana: requestedPrivileges.map(permission => actions.api.get(permission))
        });
        for (const kbPrivilege of checkPrivilegesResponse.privileges.kibana) {
          kibanaPrivileges[privilegeToApiOperation(kbPrivilege.privilege)] = kbPrivilege.authorized;
        }
      }
      for (const reservedPrivilege of requestedReservedPrivileges) {
        if (reservedPrivilege === _server.ReservedPrivilegesSet.superuser) {
          const checkSuperuserPrivilegesResponse = await checkPrivilegesWithRequest(request).globally(_constants.SUPERUSER_PRIVILEGES);
          kibanaPrivileges[_server.ReservedPrivilegesSet.superuser] = checkSuperuserPrivilegesResponse.hasAllRequested;
        }
      }
      const hasRequestedPrivilege = kbPrivilege => {
        if (typeof kbPrivilege === 'object') {
          var _kbPrivilege$allRequi, _kbPrivilege$anyRequi;
          const allRequired = (_kbPrivilege$allRequi = kbPrivilege.allRequired) !== null && _kbPrivilege$allRequi !== void 0 ? _kbPrivilege$allRequi : [];
          const anyRequired = (_kbPrivilege$anyRequi = kbPrivilege.anyRequired) !== null && _kbPrivilege$anyRequi !== void 0 ? _kbPrivilege$anyRequi : [];
          return allRequired.every(privilege => kibanaPrivileges[privilege]) && (!anyRequired.length || anyRequired.some(privilege => kibanaPrivileges[privilege]));
        }
        return kibanaPrivileges[kbPrivilege];
      };
      for (const requiredPrivilege of authz.requiredPrivileges) {
        if (!hasRequestedPrivilege(requiredPrivilege)) {
          const missingPrivileges = Object.keys(kibanaPrivileges).filter(key => !kibanaPrivileges[key]);
          const forbiddenMessage = `API [${request.route.method.toUpperCase()} ${request.url.pathname}${request.url.search}] is unauthorized for user, this action is granted by the Kibana privileges [${missingPrivileges}]`;
          logger.warn(forbiddenMessage);
          return response.forbidden({
            body: {
              message: forbiddenMessage
            }
          });
        }
      }
      return toolkit.authzResultNext(kibanaPrivileges);
    }
    const tags = request.route.options.tags;
    const tagPrefix = 'access:';
    const actionTags = tags.filter(tag => tag.startsWith(tagPrefix));

    // if there are no tags starting with "access:", just continue
    if (actionTags.length === 0) {
      return toolkit.next();
    }
    const apiActions = actionTags.map(tag => actions.api.get(tag.substring(tagPrefix.length)));
    const checkPrivileges = checkPrivilegesDynamicallyWithRequest(request);
    const checkPrivilegesResponse = await checkPrivileges({
      kibana: apiActions
    });

    // we've actually authorized the request
    if (checkPrivilegesResponse.hasAllRequested) {
      logger.debug(`User authorized for "${request.url.pathname}${request.url.search}"`);
      return toolkit.next();
    }
    logger.warn(`User not authorized for "${request.url.pathname}${request.url.search}": responding with 403`);
    return response.forbidden();
  });
}