"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defineGetAllRolesRoutes = defineGetAllRolesRoutes;
var _configSchema = require("@kbn/config-schema");
var _constants = require("../../../../common/constants");
var _authorization = require("../../../authorization");
var _errors = require("../../../errors");
var _licensed_route_handler = require("../../licensed_route_handler");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

function defineGetAllRolesRoutes({
  router,
  authz,
  getFeatures,
  subFeaturePrivilegeIterator,
  logger,
  buildFlavor
}) {
  router.versioned.get({
    path: '/api/security/role',
    access: 'public',
    summary: `Get all roles`,
    options: {
      tags: ['oas-tag:roles']
    }
  }).addVersion({
    version: _constants.API_VERSIONS.roles.public.v1,
    security: {
      authz: {
        enabled: false,
        reason: `This route delegates authorization to Core's scoped ES cluster client`
      }
    },
    validate: {
      request: {
        query: _configSchema.schema.maybe(_configSchema.schema.object({
          replaceDeprecatedPrivileges: _configSchema.schema.maybe(_configSchema.schema.boolean({
            meta: {
              description: 'If `true` and the response contains any privileges that are associated with deprecated features, they are omitted in favor of details about the appropriate replacement feature privileges.'
            }
          }))
        }))
      },
      response: {
        200: {
          description: 'Indicates a successful call.'
        }
      }
    }
  }, (0, _licensed_route_handler.createLicensedRouteHandler)(async (context, request, response) => {
    try {
      const hideReservedRoles = buildFlavor === 'serverless';
      const esClient = (await context.core).elasticsearch.client;
      const [features, elasticsearchRoles] = await Promise.all([getFeatures(), await esClient.asCurrentUser.security.getRole()]);

      // Transform elasticsearch roles into Kibana roles and return in a list sorted by the role name.
      return response.ok({
        body: Object.entries(elasticsearchRoles).map(([roleName, elasticsearchRole]) => {
          var _request$query$replac, _request$query;
          return (0, _authorization.transformElasticsearchRoleToRole)({
            features,
            subFeaturePrivilegeIterator,
            // @ts-expect-error @elastic/elasticsearch SecurityIndicesPrivileges.names expected to be string[]
            elasticsearchRole,
            name: roleName,
            application: authz.applicationName,
            logger,
            replaceDeprecatedKibanaPrivileges: (_request$query$replac = (_request$query = request.query) === null || _request$query === void 0 ? void 0 : _request$query.replaceDeprecatedPrivileges) !== null && _request$query$replac !== void 0 ? _request$query$replac : false
          });
        }).filter(role => {
          var _role$metadata;
          return !hideReservedRoles || !((_role$metadata = role.metadata) !== null && _role$metadata !== void 0 && _role$metadata._reserved);
        }).sort(_authorization.compareRolesByName)
      });
    } catch (error) {
      return response.customError((0, _errors.wrapIntoCustomErrorResponse)(error));
    }
  }));
}