"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.performRuleUpgradeRoute = void 0;
var _securitysolutionEsUtils = require("@kbn/securitysolution-es-utils");
var _zodHelpers = require("@kbn/zod-helpers");
var _prebuilt_rules = require("../../../../../../common/api/detection_engine/prebuilt_rules");
var _utils = require("../../../routes/utils");
var _aggregate_prebuilt_rule_errors = require("../../logic/aggregate_prebuilt_rule_errors");
var _perform_timelines_installation = require("../../logic/perform_timelines_installation");
var _prebuilt_rule_assets_client = require("../../logic/rule_assets/prebuilt_rule_assets_client");
var _prebuilt_rule_objects_client = require("../../logic/rule_objects/prebuilt_rule_objects_client");
var _upgrade_prebuilt_rules = require("../../logic/rule_objects/upgrade_prebuilt_rules");
var _fetch_rule_versions_triad = require("../../logic/rule_versions/fetch_rule_versions_triad");
var _constants = require("../../constants");
var _get_upgradeable_rules = require("./get_upgradeable_rules");
var _create_upgradeable_rules_payload = require("./create_upgradeable_rules_payload");
var _get_rule_groups = require("../../model/rule_groups/get_rule_groups");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const performRuleUpgradeRoute = (router, config) => {
  router.versioned.post({
    access: 'internal',
    path: _prebuilt_rules.PERFORM_RULE_UPGRADE_URL,
    security: {
      authz: {
        requiredPrivileges: ['securitySolution']
      }
    },
    options: {
      timeout: {
        idleSocket: _constants.PREBUILT_RULES_OPERATION_SOCKET_TIMEOUT_MS
      }
    }
  }).addVersion({
    version: '1',
    validate: {
      request: {
        body: (0, _zodHelpers.buildRouteValidationWithZod)(_prebuilt_rules.PerformRuleUpgradeRequestBody)
      }
    }
  }, async (context, request, response) => {
    const siemResponse = (0, _utils.buildSiemResponse)(response);
    try {
      const ctx = await context.resolve(['core', 'alerting', 'securitySolution']);
      const soClient = ctx.core.savedObjects.client;
      const rulesClient = ctx.alerting.getRulesClient();
      const detectionRulesClient = ctx.securitySolution.getDetectionRulesClient();
      const ruleAssetsClient = (0, _prebuilt_rule_assets_client.createPrebuiltRuleAssetsClient)(soClient);
      const ruleObjectsClient = (0, _prebuilt_rule_objects_client.createPrebuiltRuleObjectsClient)(rulesClient);
      const {
        mode
      } = request.body;
      const versionSpecifiers = mode === _prebuilt_rules.ModeEnum.ALL_RULES ? undefined : request.body.rules;
      const ruleTriadsMap = await (0, _fetch_rule_versions_triad.fetchRuleVersionsTriad)({
        ruleAssetsClient,
        ruleObjectsClient,
        versionSpecifiers
      });
      const ruleGroups = (0, _get_rule_groups.getRuleGroups)(ruleTriadsMap);
      const {
        upgradeableRules,
        skippedRules,
        fetchErrors
      } = (0, _get_upgradeable_rules.getUpgradeableRules)({
        rawUpgradeableRules: ruleGroups.upgradeableRules,
        currentRules: ruleGroups.currentRules,
        versionSpecifiers,
        mode
      });
      const {
        prebuiltRulesCustomizationEnabled
      } = config.experimentalFeatures;
      const {
        modifiedPrebuiltRuleAssets,
        processingErrors
      } = (0, _create_upgradeable_rules_payload.createModifiedPrebuiltRuleAssets)({
        upgradeableRules,
        requestBody: request.body,
        prebuiltRulesCustomizationEnabled
      });
      const {
        results: updatedRules,
        errors: installationErrors
      } = await (0, _upgrade_prebuilt_rules.upgradePrebuiltRules)(detectionRulesClient, modifiedPrebuiltRuleAssets);
      const ruleErrors = [...fetchErrors, ...processingErrors, ...installationErrors];
      const {
        error: timelineInstallationError
      } = await (0, _perform_timelines_installation.performTimelinesInstallation)(ctx.securitySolution);
      const allErrors = (0, _aggregate_prebuilt_rule_errors.aggregatePrebuiltRuleErrors)(ruleErrors);
      if (timelineInstallationError) {
        allErrors.push({
          message: timelineInstallationError,
          rules: []
        });
      }
      const body = {
        summary: {
          total: updatedRules.length + skippedRules.length + ruleErrors.length,
          skipped: skippedRules.length,
          succeeded: updatedRules.length,
          failed: ruleErrors.length
        },
        results: {
          updated: updatedRules.map(({
            result
          }) => result),
          skipped: skippedRules
        },
        errors: allErrors
      };
      return response.ok({
        body
      });
    } catch (err) {
      const error = (0, _securitysolutionEsUtils.transformError)(err);
      return siemResponse.error({
        body: error.message,
        statusCode: error.statusCode
      });
    }
  });
};
exports.performRuleUpgradeRoute = performRuleUpgradeRoute;