"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.reviewRuleUpgradeRoute = void 0;
var _securitysolutionEsUtils = require("@kbn/securitysolution-es-utils");
var _lodash = require("lodash");
var _prebuilt_rules = require("../../../../../../common/api/detection_engine/prebuilt_rules");
var _invariant = require("../../../../../../common/utils/invariant");
var _utils = require("../../../routes/utils");
var _calculate_rule_diff = require("../../logic/diff/calculate_rule_diff");
var _prebuilt_rule_assets_client = require("../../logic/rule_assets/prebuilt_rule_assets_client");
var _prebuilt_rule_objects_client = require("../../logic/rule_objects/prebuilt_rule_objects_client");
var _fetch_rule_versions_triad = require("../../logic/rule_versions/fetch_rule_versions_triad");
var _convert_prebuilt_rule_asset_to_rule_response = require("../../../rule_management/logic/detection_rules_client/converters/convert_prebuilt_rule_asset_to_rule_response");
var _constants = require("../../constants");
var _get_rule_groups = require("../../model/rule_groups/get_rule_groups");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const reviewRuleUpgradeRoute = router => {
  router.versioned.post({
    access: 'internal',
    path: _prebuilt_rules.REVIEW_RULE_UPGRADE_URL,
    security: {
      authz: {
        requiredPrivileges: ['securitySolution']
      }
    },
    options: {
      timeout: {
        idleSocket: _constants.PREBUILT_RULES_OPERATION_SOCKET_TIMEOUT_MS
      }
    }
  }).addVersion({
    version: '1',
    validate: {}
  }, async (context, request, response) => {
    const siemResponse = (0, _utils.buildSiemResponse)(response);
    try {
      const ctx = await context.resolve(['core', 'alerting']);
      const soClient = ctx.core.savedObjects.client;
      const rulesClient = ctx.alerting.getRulesClient();
      const ruleAssetsClient = (0, _prebuilt_rule_assets_client.createPrebuiltRuleAssetsClient)(soClient);
      const ruleObjectsClient = (0, _prebuilt_rule_objects_client.createPrebuiltRuleObjectsClient)(rulesClient);
      const ruleVersionsMap = await (0, _fetch_rule_versions_triad.fetchRuleVersionsTriad)({
        ruleAssetsClient,
        ruleObjectsClient
      });
      const {
        upgradeableRules
      } = (0, _get_rule_groups.getRuleGroups)(ruleVersionsMap);
      const ruleDiffCalculationResults = upgradeableRules.map(({
        current
      }) => {
        const ruleVersions = ruleVersionsMap.get(current.rule_id);
        (0, _invariant.invariant)(ruleVersions != null, 'ruleVersions not found');
        return (0, _calculate_rule_diff.calculateRuleDiff)(ruleVersions);
      });
      const body = {
        stats: calculateRuleStats(ruleDiffCalculationResults),
        rules: calculateRuleInfos(ruleDiffCalculationResults)
      };
      return response.ok({
        body
      });
    } catch (err) {
      const error = (0, _securitysolutionEsUtils.transformError)(err);
      return siemResponse.error({
        body: error.message,
        statusCode: error.statusCode
      });
    }
  });
};
exports.reviewRuleUpgradeRoute = reviewRuleUpgradeRoute;
const calculateRuleStats = results => {
  const allTags = new Set();
  const stats = results.reduce((acc, result) => {
    var _result$ruleVersions$;
    acc.num_rules_to_upgrade_total += 1;
    if (result.ruleDiff.num_fields_with_conflicts > 0) {
      acc.num_rules_with_conflicts += 1;
    }
    if (result.ruleDiff.num_fields_with_non_solvable_conflicts > 0) {
      acc.num_rules_with_non_solvable_conflicts += 1;
    }
    (_result$ruleVersions$ = result.ruleVersions.input.current) === null || _result$ruleVersions$ === void 0 ? void 0 : _result$ruleVersions$.tags.forEach(tag => allTags.add(tag));
    return acc;
  }, {
    num_rules_to_upgrade_total: 0,
    num_rules_with_conflicts: 0,
    num_rules_with_non_solvable_conflicts: 0
  });
  return {
    ...stats,
    tags: Array.from(allTags)
  };
};
const calculateRuleInfos = results => {
  return results.map(result => {
    const {
      ruleDiff,
      ruleVersions
    } = result;
    const installedCurrentVersion = ruleVersions.input.current;
    const targetVersion = ruleVersions.input.target;
    (0, _invariant.invariant)(installedCurrentVersion != null, 'installedCurrentVersion not found');
    (0, _invariant.invariant)(targetVersion != null, 'targetVersion not found');
    const targetRule = {
      ...(0, _convert_prebuilt_rule_asset_to_rule_response.convertPrebuiltRuleAssetToRuleResponse)(targetVersion),
      id: installedCurrentVersion.id,
      revision: installedCurrentVersion.revision + 1,
      created_at: installedCurrentVersion.created_at,
      created_by: installedCurrentVersion.created_by,
      updated_at: new Date().toISOString(),
      updated_by: installedCurrentVersion.updated_by
    };
    return {
      id: installedCurrentVersion.id,
      rule_id: installedCurrentVersion.rule_id,
      revision: installedCurrentVersion.revision,
      current_rule: installedCurrentVersion,
      target_rule: targetRule,
      diff: {
        fields: (0, _lodash.pickBy)(ruleDiff.fields, fieldDiff => fieldDiff.diff_outcome !== _prebuilt_rules.ThreeWayDiffOutcome.StockValueNoUpdate && fieldDiff.diff_outcome !== _prebuilt_rules.ThreeWayDiffOutcome.MissingBaseNoUpdate),
        num_fields_with_updates: ruleDiff.num_fields_with_updates,
        num_fields_with_conflicts: ruleDiff.num_fields_with_conflicts,
        num_fields_with_non_solvable_conflicts: ruleDiff.num_fields_with_non_solvable_conflicts
      }
    };
  });
};