"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.multiLineStringDiffAlgorithm = void 0;
var _nodeDiff = require("node-diff3");
var _utility_types = require("../../../../../../../../common/utility_types");
var _prebuilt_rules = require("../../../../../../../../common/api/detection_engine/prebuilt_rules");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

/**
 * Diff algorithm used for string fields that contain multiple lines
 */
const multiLineStringDiffAlgorithm = versions => {
  const {
    base_version: baseVersion,
    current_version: currentVersion,
    target_version: targetVersion
  } = versions;
  const diffOutcome = (0, _prebuilt_rules.determineDiffOutcome)(baseVersion, currentVersion, targetVersion);
  const valueCanUpdate = (0, _prebuilt_rules.determineIfValueCanUpdate)(diffOutcome);
  const hasBaseVersion = baseVersion !== _prebuilt_rules.MissingVersion;
  const {
    mergeOutcome,
    conflict,
    mergedVersion
  } = mergeVersions({
    baseVersion: hasBaseVersion ? baseVersion : undefined,
    currentVersion,
    targetVersion,
    diffOutcome
  });
  return {
    has_base_version: hasBaseVersion,
    base_version: hasBaseVersion ? baseVersion : undefined,
    current_version: currentVersion,
    target_version: targetVersion,
    merged_version: mergedVersion,
    merge_outcome: mergeOutcome,
    diff_outcome: diffOutcome,
    conflict,
    has_update: valueCanUpdate
  };
};
exports.multiLineStringDiffAlgorithm = multiLineStringDiffAlgorithm;
const mergeVersions = ({
  baseVersion,
  currentVersion,
  targetVersion,
  diffOutcome
}) => {
  switch (diffOutcome) {
    // Scenario -AA is treated as scenario AAA:
    // https://github.com/elastic/kibana/pull/184889#discussion_r1636421293
    case _prebuilt_rules.ThreeWayDiffOutcome.MissingBaseNoUpdate:
    case _prebuilt_rules.ThreeWayDiffOutcome.StockValueNoUpdate:
    case _prebuilt_rules.ThreeWayDiffOutcome.CustomizedValueNoUpdate:
    case _prebuilt_rules.ThreeWayDiffOutcome.CustomizedValueSameUpdate:
      return {
        conflict: _prebuilt_rules.ThreeWayDiffConflict.NONE,
        mergedVersion: currentVersion,
        mergeOutcome: _prebuilt_rules.ThreeWayMergeOutcome.Current
      };
    case _prebuilt_rules.ThreeWayDiffOutcome.StockValueCanUpdate:
      {
        return {
          conflict: _prebuilt_rules.ThreeWayDiffConflict.NONE,
          mergedVersion: targetVersion,
          mergeOutcome: _prebuilt_rules.ThreeWayMergeOutcome.Target
        };
      }
    case _prebuilt_rules.ThreeWayDiffOutcome.CustomizedValueCanUpdate:
      {
        // TS does not realize that in ABC scenario, baseVersion cannot be missing
        // Missing baseVersion scenarios were handled as -AA and -AB.
        const mergedVersion = (0, _nodeDiff.merge)(currentVersion, baseVersion !== null && baseVersion !== void 0 ? baseVersion : '', targetVersion, {
          stringSeparator: /(\r\n|\n|\r)/g // Separates strings by new lines
        });
        return mergedVersion.conflict ? {
          conflict: _prebuilt_rules.ThreeWayDiffConflict.NON_SOLVABLE,
          mergedVersion: currentVersion,
          mergeOutcome: _prebuilt_rules.ThreeWayMergeOutcome.Current
        } : {
          conflict: _prebuilt_rules.ThreeWayDiffConflict.SOLVABLE,
          mergedVersion: mergedVersion.result.join(''),
          mergeOutcome: _prebuilt_rules.ThreeWayMergeOutcome.Merged
        };
      }

    // Scenario -AB is treated as scenario ABC, but marked as
    // SOLVABLE, and returns the target version as the merged version
    // https://github.com/elastic/kibana/pull/184889#discussion_r1636421293
    case _prebuilt_rules.ThreeWayDiffOutcome.MissingBaseCanUpdate:
      {
        return {
          mergedVersion: targetVersion,
          mergeOutcome: _prebuilt_rules.ThreeWayMergeOutcome.Target,
          conflict: _prebuilt_rules.ThreeWayDiffConflict.SOLVABLE
        };
      }
    default:
      return (0, _utility_types.assertUnreachable)(diffOutcome);
  }
};