"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFieldRetentionEnrichPolicyName = exports.executeFieldRetentionEnrichPolicy = exports.deleteFieldRetentionEnrichPolicy = exports.createFieldRetentionEnrichPolicy = void 0;
var _utils = require("../utils");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const getFieldRetentionEnrichPolicyName = ({
  namespace,
  entityType,
  version
}) => {
  return `entity_store_field_retention_${entityType}_${namespace}_v${version}`;
};
exports.getFieldRetentionEnrichPolicyName = getFieldRetentionEnrichPolicyName;
const getFieldRetentionEnrichPolicy = unitedDefinition => {
  const {
    namespace,
    entityType,
    fieldRetentionDefinition
  } = unitedDefinition;
  return {
    name: getFieldRetentionEnrichPolicyName(unitedDefinition),
    match: {
      indices: (0, _utils.getEntitiesIndexName)(entityType, namespace),
      match_field: fieldRetentionDefinition.matchField,
      enrich_fields: fieldRetentionDefinition.fields.map(({
        field
      }) => field)
    }
  };
};
const createFieldRetentionEnrichPolicy = async ({
  esClient,
  unitedDefinition
}) => {
  const policy = getFieldRetentionEnrichPolicy(unitedDefinition);
  return esClient.enrich.putPolicy(policy);
};
exports.createFieldRetentionEnrichPolicy = createFieldRetentionEnrichPolicy;
const executeFieldRetentionEnrichPolicy = async ({
  esClient,
  unitedDefinition,
  logger
}) => {
  const name = getFieldRetentionEnrichPolicyName(unitedDefinition);
  try {
    await esClient.enrich.executePolicy({
      name
    });
    return {
      executed: true
    };
  } catch (e) {
    if (e.statusCode === 404) {
      return {
        executed: false
      };
    }
    logger.error(`Error executing field retention enrich policy for ${unitedDefinition.entityType}: ${e.message}`);
    throw e;
  }
};
exports.executeFieldRetentionEnrichPolicy = executeFieldRetentionEnrichPolicy;
const deleteFieldRetentionEnrichPolicy = async ({
  unitedDefinition,
  esClient,
  logger,
  attempts = 5,
  delayMs = 2000
}) => {
  const name = getFieldRetentionEnrichPolicyName(unitedDefinition);
  let currentAttempt = 1;
  while (currentAttempt <= attempts) {
    try {
      await esClient.enrich.deletePolicy({
        name
      }, {
        ignore: [404]
      });
      return;
    } catch (e) {
      // a 429 status code indicates that the enrich policy is being executed
      if (currentAttempt === attempts || e.statusCode !== 429) {
        logger.error(`Error deleting enrich policy ${name}: ${e.message} after ${currentAttempt} attempts`);
        throw e;
      }
      logger.info(`Enrich policy ${name} is being executed, waiting for it to finish before deleting`);
      await new Promise(resolve => setTimeout(resolve, delayMs));
      currentAttempt++;
    }
  }
};
exports.deleteFieldRetentionEnrichPolicy = deleteFieldRetentionEnrichPolicy;