"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deletePlatformPipeline = exports.createPlatformPipeline = exports.ENRICH_FIELD = void 0;
var _ingest_processor_steps = require("./ingest_processor_steps");
var _utils = require("../utils");
var _enrich_policy = require("./enrich_policy");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const getPlatformPipelineId = definition => {
  return `${definition.id}-latest@platform`;
};

// the field that the enrich processor writes to
const ENRICH_FIELD = exports.ENRICH_FIELD = 'historical';

/**
 * Builds the ingest pipeline for the field retention policy.
 * Broadly the pipeline enriches the entity with the field retention enrich policy,
 * then applies the field retention policy to the entity fields, and finally removes
 * the enrich field and any empty fields.
 *
 * While developing, be sure to set debugMode to true this will keep the enrich field
 * and the context field in the document to help with debugging.
 */
const buildIngestPipeline = ({
  version,
  fieldRetentionDefinition,
  allEntityFields,
  debugMode,
  namespace
}) => {
  const {
    entityType,
    matchField
  } = fieldRetentionDefinition;
  const enrichPolicyName = (0, _enrich_policy.getFieldRetentionEnrichPolicyName)({
    namespace,
    entityType,
    version
  });
  return [...(debugMode ? [(0, _ingest_processor_steps.debugDeepCopyContextStep)()] : []), {
    enrich: {
      policy_name: enrichPolicyName,
      field: matchField,
      target_field: ENRICH_FIELD
    }
  }, {
    set: {
      field: '@timestamp',
      value: '{{entity.last_seen_timestamp}}'
    }
  }, {
    set: {
      field: 'entity.name',
      value: `{{${(0, _utils.getIdentityFieldForEntityType)(entityType)}}}`
    }
  }, ...(0, _ingest_processor_steps.getDotExpanderSteps)(allEntityFields), ...(0, _ingest_processor_steps.retentionDefinitionToIngestProcessorSteps)(fieldRetentionDefinition, {
    enrichField: ENRICH_FIELD
  }), ...(0, _ingest_processor_steps.getRemoveEmptyFieldSteps)([...allEntityFields, 'asset', `${entityType}.risk`]), (0, _ingest_processor_steps.removeEntityDefinitionFieldsStep)(), ...(!debugMode ? [{
    remove: {
      ignore_failure: true,
      field: ENRICH_FIELD
    }
  }] : [])];
};

// developing the pipeline is a bit tricky, so we have a debug mode
// set  xpack.securitySolution.entityAnalytics.entityStore.developer.pipelineDebugMode
// to true to keep the enrich field and the context field in the document to help with debugging.
const createPlatformPipeline = async ({
  unitedDefinition,
  logger,
  esClient,
  debugMode
}) => {
  const {
    fieldRetentionDefinition,
    entityManagerDefinition
  } = unitedDefinition;
  const allEntityFields = ((entityManagerDefinition === null || entityManagerDefinition === void 0 ? void 0 : entityManagerDefinition.metadata) || []).map(m => {
    if (typeof m === 'string') {
      return m;
    }
    return m.destination;
  });
  const pipeline = {
    id: getPlatformPipelineId(entityManagerDefinition),
    body: {
      _meta: {
        managed_by: 'entity_store',
        managed: true
      },
      description: `Ingest pipeline for entity definition ${entityManagerDefinition.id}`,
      processors: buildIngestPipeline({
        namespace: unitedDefinition.namespace,
        version: unitedDefinition.version,
        fieldRetentionDefinition,
        allEntityFields,
        debugMode
      })
    }
  };
  logger.debug(`Attempting to create pipeline: ${JSON.stringify(pipeline)}`);
  await esClient.ingest.putPipeline(pipeline);
};
exports.createPlatformPipeline = createPlatformPipeline;
const deletePlatformPipeline = ({
  unitedDefinition,
  logger,
  esClient
}) => {
  const pipelineId = getPlatformPipelineId(unitedDefinition.entityManagerDefinition);
  logger.debug(`Attempting to delete pipeline: ${pipelineId}`);
  return esClient.ingest.deletePipeline({
    id: pipelineId
  }, {
    ignore: [404]
  });
};
exports.deletePlatformPipeline = deletePlatformPipeline;