"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EntityStoreDataClient = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _entity_client = require("@kbn/entityManager-plugin/server/lib/entity_client");
var _fp = require("lodash/fp");
var _moment = _interopRequireDefault(require("moment"));
var _engine_descriptor = require("./saved_object/engine_descriptor");
var _constants = require("./constants");
var _asset_criticality_migration_client = require("../asset_criticality/asset_criticality_migration_client");
var _united_entity_definitions = require("./united_entity_definitions");
var _task = require("./task");
var _elasticsearch_assets = require("./elasticsearch_assets");
var _risk_score_data_client = require("../risk_score/risk_score_data_client");
var _utils = require("./utils");
var _actions = require("./auditing/actions");
var _resources = require("./auditing/resources");
var _audit = require("../audit");
var _events = require("../../telemetry/event_based/events");
var _constants2 = require("../asset_criticality/constants");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

class EntityStoreDataClient {
  constructor(options) {
    (0, _defineProperty2.default)(this, "engineClient", void 0);
    (0, _defineProperty2.default)(this, "assetCriticalityMigrationClient", void 0);
    (0, _defineProperty2.default)(this, "entityClient", void 0);
    (0, _defineProperty2.default)(this, "riskScoreDataClient", void 0);
    (0, _defineProperty2.default)(this, "esClient", void 0);
    this.options = options;
    const {
      clusterClient,
      logger,
      soClient,
      auditLogger,
      kibanaVersion,
      namespace
    } = options;
    this.esClient = clusterClient.asCurrentUser;
    this.entityClient = new _entity_client.EntityClient({
      esClient: this.esClient,
      soClient,
      logger
    });
    this.engineClient = new _engine_descriptor.EngineDescriptorClient({
      soClient,
      namespace
    });
    this.assetCriticalityMigrationClient = new _asset_criticality_migration_client.AssetCriticalityEcsMigrationClient({
      esClient: this.esClient,
      logger,
      auditLogger
    });
    this.riskScoreDataClient = new _risk_score_data_client.RiskScoreDataClient({
      soClient,
      esClient: this.esClient,
      logger,
      namespace,
      kibanaVersion
    });
  }
  async init(entityType, {
    indexPattern = '',
    filter = '',
    fieldHistoryLength = 10
  }, {
    pipelineDebugMode = false
  } = {}) {
    if (!this.options.taskManager) {
      throw new Error('Task Manager is not available');
    }
    const {
      config
    } = this.options;
    await this.riskScoreDataClient.createRiskScoreLatestIndex();
    const requiresMigration = await this.assetCriticalityMigrationClient.isEcsDataMigrationRequired();
    if (requiresMigration) {
      throw new Error('Asset criticality data migration is required before initializing entity store. If this error persists, please restart Kibana.');
    }
    this.log('info', entityType, `Initializing entity store`);
    this.audit(_actions.EntityEngineActions.INIT, _resources.EntityStoreResource.ENTITY_ENGINE, entityType, 'Initializing entity engine');
    const descriptor = await this.engineClient.init(entityType, {
      filter,
      fieldHistoryLength,
      indexPattern
    });
    this.log('debug', entityType, `Initialized engine saved object`);
    this.asyncSetup(entityType, fieldHistoryLength, this.options.taskManager, indexPattern, filter, config, pipelineDebugMode).catch(e => this.log('error', entityType, `Error during async setup of entity store: ${e.message}`));
    return descriptor;
  }
  async asyncSetup(entityType, fieldHistoryLength, taskManager, indexPattern, filter, config, pipelineDebugMode) {
    const setupStartTime = (0, _moment.default)().utc().toISOString();
    const {
      logger,
      namespace,
      appClient,
      dataViewsService
    } = this.options;
    const indexPatterns = await (0, _utils.buildIndexPatterns)(namespace, appClient, dataViewsService);
    const unitedDefinition = (0, _united_entity_definitions.getUnitedEntityDefinition)({
      indexPatterns,
      entityType,
      namespace,
      fieldHistoryLength,
      syncDelay: `${config.syncDelay.asSeconds()}s`,
      frequency: `${config.frequency.asSeconds()}s`
    });
    const {
      entityManagerDefinition
    } = unitedDefinition;
    try {
      var _this$options$telemet;
      // clean up any existing entity store
      await this.delete(entityType, taskManager, {
        deleteData: false,
        deleteEngine: false
      });

      // set up the entity manager definition
      await this.entityClient.createEntityDefinition({
        definition: {
          ...entityManagerDefinition,
          filter,
          indexPatterns: indexPattern ? [...entityManagerDefinition.indexPatterns, ...indexPattern.split(',')] : entityManagerDefinition.indexPatterns
        },
        installOnly: true
      });
      this.log(`debug`, entityType, `Created entity definition`);

      // the index must be in place with the correct mapping before the enrich policy is created
      // this is because the enrich policy will fail if the index does not exist with the correct fields
      await (0, _elasticsearch_assets.createEntityIndexComponentTemplate)({
        unitedDefinition,
        esClient: this.esClient
      });
      this.log(`debug`, entityType, `Created entity index component template`);
      await (0, _elasticsearch_assets.createEntityIndex)({
        entityType,
        esClient: this.esClient,
        namespace,
        logger
      });
      this.log(`debug`, entityType, `Created entity index`);

      // we must create and execute the enrich policy before the pipeline is created
      // this is because the pipeline will fail if the enrich index does not exist
      await (0, _elasticsearch_assets.createFieldRetentionEnrichPolicy)({
        unitedDefinition,
        esClient: this.esClient
      });
      this.log(`debug`, entityType, `Created field retention enrich policy`);
      await (0, _elasticsearch_assets.executeFieldRetentionEnrichPolicy)({
        unitedDefinition,
        esClient: this.esClient,
        logger
      });
      this.log(`debug`, entityType, `Executed field retention enrich policy`);
      await (0, _elasticsearch_assets.createPlatformPipeline)({
        debugMode: pipelineDebugMode,
        unitedDefinition,
        logger,
        esClient: this.esClient
      });
      this.log(`debug`, entityType, `Created @platform pipeline`);

      // finally start the entity definition now that everything is in place
      const updated = await this.start(entityType, {
        force: true
      });

      // the task will execute the enrich policy on a schedule
      await (0, _task.startEntityStoreFieldRetentionEnrichTask)({
        namespace,
        logger,
        taskManager
      });
      this.log(`debug`, entityType, `Started entity store field retention enrich task`);
      this.log(`info`, entityType, `Entity store initialized`);
      const setupEndTime = (0, _moment.default)().utc().toISOString();
      const duration = (0, _moment.default)(setupEndTime).diff((0, _moment.default)(setupStartTime), 'seconds');
      (_this$options$telemet = this.options.telemetry) === null || _this$options$telemet === void 0 ? void 0 : _this$options$telemet.reportEvent(_events.ENTITY_ENGINE_INITIALIZATION_EVENT.eventType, {
        duration
      });
      return updated;
    } catch (err) {
      var _this$options$telemet2;
      this.log(`error`, entityType, `Error initializing entity store: ${err.message}`);
      this.audit(_actions.EntityEngineActions.INIT, _resources.EntityStoreResource.ENTITY_ENGINE, entityType, 'Failed to initialize entity engine resources', err);
      (_this$options$telemet2 = this.options.telemetry) === null || _this$options$telemet2 === void 0 ? void 0 : _this$options$telemet2.reportEvent(_events.ENTITY_ENGINE_RESOURCE_INIT_FAILURE_EVENT.eventType, {
        error: err.message
      });
      await this.engineClient.update(entityType, {
        status: _constants.ENGINE_STATUS.ERROR,
        error: {
          message: err.message,
          stack: err.stack,
          action: 'init'
        }
      });
      await this.delete(entityType, taskManager, {
        deleteData: true,
        deleteEngine: false
      });
    }
  }
  async getExistingEntityDefinition(entityType) {
    const entityDefinitionId = (0, _utils.buildEntityDefinitionId)(entityType, this.options.namespace);
    const {
      definitions: [definition]
    } = await this.entityClient.getEntityDefinitions({
      id: entityDefinitionId
    });
    if (!definition) {
      throw new Error(`Unable to find entity definition for ${entityType}`);
    }
    return definition;
  }
  async start(entityType, options) {
    const {
      namespace
    } = this.options;
    const descriptor = await this.engineClient.get(entityType);
    if (!(options !== null && options !== void 0 && options.force) && descriptor.status !== _constants.ENGINE_STATUS.STOPPED) {
      throw new Error(`In namespace ${namespace}: Cannot start Entity engine for ${entityType} when current status is: ${descriptor.status}`);
    }
    this.log('info', entityType, `Starting entity store`);

    // startEntityDefinition requires more fields than the engine descriptor
    // provides so we need to fetch the full entity definition
    const fullEntityDefinition = await this.getExistingEntityDefinition(entityType);
    this.audit(_actions.EntityEngineActions.START, _resources.EntityStoreResource.ENTITY_DEFINITION, entityType, 'Starting entity definition');
    await this.entityClient.startEntityDefinition(fullEntityDefinition);
    this.log('debug', entityType, `Started entity definition`);
    return this.engineClient.updateStatus(entityType, _constants.ENGINE_STATUS.STARTED);
  }
  async stop(entityType) {
    const {
      namespace
    } = this.options;
    const descriptor = await this.engineClient.get(entityType);
    if (descriptor.status !== _constants.ENGINE_STATUS.STARTED) {
      throw new Error(`In namespace ${namespace}: Cannot stop Entity engine for ${entityType} when current status is: ${descriptor.status}`);
    }
    this.log('info', entityType, `Stopping entity store`);

    // stopEntityDefinition requires more fields than the engine descriptor
    // provides so we need to fetch the full entity definition
    const fullEntityDefinition = await this.getExistingEntityDefinition(entityType);
    this.audit(_actions.EntityEngineActions.STOP, _resources.EntityStoreResource.ENTITY_DEFINITION, entityType, 'Stopping entity definition');
    await this.entityClient.stopEntityDefinition(fullEntityDefinition);
    this.log('debug', entityType, `Stopped entity definition`);
    return this.engineClient.updateStatus(entityType, _constants.ENGINE_STATUS.STOPPED);
  }
  async get(entityType) {
    return this.engineClient.get(entityType);
  }
  async list() {
    return this.engineClient.list();
  }
  async delete(entityType, taskManager, options = {
    deleteData: false,
    deleteEngine: true
  }) {
    var _descriptor$fieldHist;
    const {
      namespace,
      logger,
      appClient,
      dataViewsService,
      config
    } = this.options;
    const {
      deleteData,
      deleteEngine
    } = options;
    const descriptor = await this.engineClient.maybeGet(entityType);
    const indexPatterns = await (0, _utils.buildIndexPatterns)(namespace, appClient, dataViewsService);

    // TODO delete unitedDefinition from this method. we only need the id for deletion
    const unitedDefinition = (0, _united_entity_definitions.getUnitedEntityDefinition)({
      indexPatterns,
      entityType,
      namespace: this.options.namespace,
      fieldHistoryLength: (_descriptor$fieldHist = descriptor === null || descriptor === void 0 ? void 0 : descriptor.fieldHistoryLength) !== null && _descriptor$fieldHist !== void 0 ? _descriptor$fieldHist : 10,
      syncDelay: `${config.syncDelay.asSeconds()}s`,
      frequency: `${config.frequency.asSeconds()}s`
    });
    const {
      entityManagerDefinition
    } = unitedDefinition;
    this.log('info', entityType, `Deleting entity store`);
    this.audit(_actions.EntityEngineActions.DELETE, _resources.EntityStoreResource.ENTITY_ENGINE, entityType, 'Deleting entity engine');
    try {
      await this.entityClient.deleteEntityDefinition({
        id: entityManagerDefinition.id,
        deleteData
      })
      // Swallowing the error as it is expected to fail if no entity definition exists
      .catch(e => this.log(`warn`, entityType, `Error deleting entity definition: ${e.message}`));
      this.log('debug', entityType, `Deleted entity definition`);
      await (0, _elasticsearch_assets.deleteEntityIndexComponentTemplate)({
        unitedDefinition,
        esClient: this.esClient
      });
      this.log('debug', entityType, `Deleted entity index component template`);
      await (0, _elasticsearch_assets.deletePlatformPipeline)({
        unitedDefinition,
        logger,
        esClient: this.esClient
      });
      this.log('debug', entityType, `Deleted platform pipeline`);
      await (0, _elasticsearch_assets.deleteFieldRetentionEnrichPolicy)({
        unitedDefinition,
        esClient: this.esClient,
        logger
      });
      this.log('debug', entityType, `Deleted field retention enrich policy`);
      if (deleteData) {
        await (0, _elasticsearch_assets.deleteEntityIndex)({
          entityType,
          esClient: this.esClient,
          namespace,
          logger
        });
        this.log('debug', entityType, `Deleted entity index`);
      }
      if (descriptor && deleteEngine) {
        await this.engineClient.delete(entityType);
      }
      // if the last engine then stop the task
      const {
        engines
      } = await this.engineClient.list();
      if (engines.length === 0) {
        await (0, _task.removeEntityStoreFieldRetentionEnrichTask)({
          namespace,
          logger,
          taskManager
        });
        this.log('debug', entityType, `Deleted entity store field retention enrich task`);
      }
      logger.info(`[Entity Store] In namespace ${namespace}: Deleted store for ${entityType}`);
      return {
        deleted: true
      };
    } catch (err) {
      this.log(`error`, entityType, `Error deleting entity store: ${err.message}`);
      this.audit(_actions.EntityEngineActions.DELETE, _resources.EntityStoreResource.ENTITY_ENGINE, entityType, 'Failed to delete entity engine', err);
      throw err;
    }
  }
  async searchEntities(params) {
    var _hits$total$value, _hits$total;
    const {
      page,
      perPage,
      sortField,
      sortOrder,
      filterQuery,
      entityTypes
    } = params;
    const index = entityTypes.map(type => (0, _utils.getEntitiesIndexName)(type, this.options.namespace));
    const from = (page - 1) * perPage;
    const sort = sortField ? [{
      [sortField]: sortOrder
    }] : undefined;
    const query = filterQuery ? JSON.parse(filterQuery) : undefined;
    const response = await this.esClient.search({
      index,
      query,
      size: Math.min(perPage, _constants.MAX_SEARCH_RESPONSE_SIZE),
      from,
      sort,
      ignore_unavailable: true
    });
    const {
      hits
    } = response;
    const total = typeof hits.total === 'number' ? hits.total : (_hits$total$value = (_hits$total = hits.total) === null || _hits$total === void 0 ? void 0 : _hits$total.value) !== null && _hits$total$value !== void 0 ? _hits$total$value : 0;
    const records = hits.hits.map(hit => {
      const {
        asset,
        ...source
      } = hit._source;
      const assetOverwrite = asset && asset.criticality !== _constants2.CRITICALITY_VALUES.DELETED ? {
        asset: {
          criticality: asset.criticality
        }
      } : {};
      return {
        ...source,
        ...assetOverwrite
      };
    });
    const inspect = {
      dsl: [JSON.stringify({
        index,
        body: query
      }, null, 2)],
      response: [JSON.stringify(response, null, 2)]
    };
    return {
      records,
      total,
      inspect
    };
  }
  async applyDataViewIndices() {
    const {
      logger
    } = this.options;
    logger.info(`In namespace ${this.options.namespace}: Applying data view indices to the entity store`);
    const {
      engines
    } = await this.engineClient.list();
    const updateDefinitionPromises = await engines.map(async engine => {
      const originalStatus = engine.status;
      const id = (0, _utils.buildEntityDefinitionId)(engine.type, this.options.namespace);
      const definition = await this.getExistingEntityDefinition(engine.type);
      if (originalStatus === _constants.ENGINE_STATUS.INSTALLING || originalStatus === _constants.ENGINE_STATUS.UPDATING) {
        throw new Error(`Error updating entity store: There are changes already in progress for engine ${id}`);
      }
      const indexPatterns = await (0, _utils.buildIndexPatterns)(this.options.namespace, this.options.appClient, this.options.dataViewsService);

      // Skip update if index patterns are the same
      if ((0, _fp.isEqual)(definition.indexPatterns, indexPatterns)) {
        return {
          type: engine.type,
          changes: {}
        };
      }

      // Update savedObject status
      await this.engineClient.updateStatus(engine.type, _constants.ENGINE_STATUS.UPDATING);
      try {
        // Update entity manager definition
        await this.entityClient.updateEntityDefinition({
          id,
          definitionUpdate: {
            ...definition,
            indexPatterns
          }
        });

        // Restore the savedObject status and set the new index pattern
        await this.engineClient.updateStatus(engine.type, originalStatus);
        return {
          type: engine.type,
          changes: {
            indexPatterns
          }
        };
      } catch (error) {
        // Rollback the engine initial status when the update fails
        await this.engineClient.updateStatus(engine.type, originalStatus);
        throw error;
      }
    });
    const updatedDefinitions = await Promise.allSettled(updateDefinitionPromises);
    const updateErrors = updatedDefinitions.filter(_utils.isPromiseRejected).map(result => result.reason);
    const updateSuccesses = updatedDefinitions.filter(_utils.isPromiseFulfilled).map(result => result.value);
    return {
      successes: updateSuccesses,
      errors: updateErrors
    };
  }
  log(level, entityType, msg) {
    this.options.logger[level](`[Entity Engine] [entity.${entityType}] [namespace: ${this.options.namespace}] ${msg}`);
  }
  audit(action, resource, entityType, msg, error) {
    var _this$options$auditLo;
    // NOTE: Excluding errors, all auditing events are currently WRITE events, meaning the outcome is always UNKNOWN.
    // This may change in the future, depending on the audit action.
    const outcome = error ? _audit.AUDIT_OUTCOME.FAILURE : _audit.AUDIT_OUTCOME.UNKNOWN;
    const type = action === _actions.EntityEngineActions.CREATE ? _audit.AUDIT_TYPE.CREATION : _actions.EntityEngineActions.DELETE ? _audit.AUDIT_TYPE.DELETION : _audit.AUDIT_TYPE.CHANGE;
    const category = _audit.AUDIT_CATEGORY.DATABASE;
    const message = error ? `${msg}: ${error.message}` : msg;
    const event = {
      message: `[Entity Engine] [entity.${entityType}] ${message}`,
      event: {
        action: `${action}_${entityType}_${resource}`,
        category,
        outcome,
        type
      }
    };
    return (_this$options$auditLo = this.options.auditLogger) === null || _this$options$auditLo === void 0 ? void 0 : _this$options$auditLo.log(event);
  }
}
exports.EntityStoreDataClient = EntityStoreDataClient;