"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.startEntityStoreFieldRetentionEnrichTask = exports.runTask = exports.removeEntityStoreFieldRetentionEnrichTask = exports.registerEntityStoreFieldRetentionEnrichTask = void 0;
var _moment = _interopRequireDefault(require("moment"));
var _server = require("@kbn/core/server");
var _state = require("./state");
var _constants = require("./constants");
var _united_entity_definitions = require("../united_entity_definitions");
var _elasticsearch_assets = require("../elasticsearch_assets");
var _utils = require("../utils");
var _events = require("../../../telemetry/event_based/events");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const logFactory = (logger, taskId) => message => logger.info(`[Entity Store] [task ${taskId}]: ${message}`);
const debugLogFactory = (logger, taskId) => message => logger.debug(`[Entity Store] [task ${taskId}]: ${message}`);
const getTaskName = () => _constants.TYPE;
const getTaskId = namespace => `${_constants.TYPE}:${namespace}:${_constants.VERSION}`;
const registerEntityStoreFieldRetentionEnrichTask = ({
  getStartServices,
  logger,
  telemetry,
  taskManager
}) => {
  if (!taskManager) {
    logger.info('[Entity Store]  Task Manager is unavailable; skipping entity store enrich policy registration.');
    return;
  }
  const executeEnrichPolicy = async (namespace, entityType) => {
    const [coreStart, _] = await getStartServices();
    const esClient = coreStart.elasticsearch.client.asInternalUser;
    const unitedDefinitionVersion = (0, _united_entity_definitions.getUnitedEntityDefinitionVersion)(entityType);
    return (0, _elasticsearch_assets.executeFieldRetentionEnrichPolicy)({
      unitedDefinition: {
        namespace,
        entityType,
        version: unitedDefinitionVersion
      },
      esClient,
      logger
    });
  };
  const getStoreSize = async index => {
    const [coreStart] = await getStartServices();
    const esClient = coreStart.elasticsearch.client.asInternalUser;
    const {
      count
    } = await esClient.count({
      index
    });
    return count;
  };
  taskManager.registerTaskDefinitions({
    [getTaskName()]: {
      title: 'Entity Analytics Entity Store - Execute Enrich Policy Task',
      timeout: _constants.TIMEOUT,
      stateSchemaByVersion: _state.stateSchemaByVersion,
      createTaskRunner: createTaskRunnerFactory({
        logger,
        telemetry,
        getStoreSize,
        executeEnrichPolicy
      })
    }
  });
};
exports.registerEntityStoreFieldRetentionEnrichTask = registerEntityStoreFieldRetentionEnrichTask;
const startEntityStoreFieldRetentionEnrichTask = async ({
  logger,
  namespace,
  taskManager
}) => {
  const taskId = getTaskId(namespace);
  const log = logFactory(logger, taskId);
  log('starting task');
  log('attempting to schedule');
  try {
    await taskManager.ensureScheduled({
      id: taskId,
      taskType: getTaskName(),
      scope: _constants.SCOPE,
      schedule: {
        interval: _constants.INTERVAL
      },
      state: {
        ..._state.defaultState,
        namespace
      },
      params: {
        version: _constants.VERSION
      }
    });
  } catch (e) {
    logger.warn(`[Entity Store]  [task ${taskId}]: error scheduling task, received ${e.message}`);
    throw e;
  }
};
exports.startEntityStoreFieldRetentionEnrichTask = startEntityStoreFieldRetentionEnrichTask;
const removeEntityStoreFieldRetentionEnrichTask = async ({
  logger,
  namespace,
  taskManager
}) => {
  try {
    await taskManager.remove(getTaskId(namespace));
    logger.info(`[Entity Store]  Removed entity store enrich policy task for namespace ${namespace}`);
  } catch (err) {
    if (!_server.SavedObjectsErrorHelpers.isNotFoundError(err)) {
      logger.error(`[Entity Store]  Failed to remove  entity store enrich policy task: ${err.message}`);
      throw err;
    }
  }
};
exports.removeEntityStoreFieldRetentionEnrichTask = removeEntityStoreFieldRetentionEnrichTask;
const runTask = async ({
  executeEnrichPolicy,
  getStoreSize,
  isCancelled,
  logger,
  taskInstance,
  telemetry
}) => {
  const state = taskInstance.state;
  const taskId = taskInstance.id;
  const log = logFactory(logger, taskId);
  const debugLog = debugLogFactory(logger, taskId);
  try {
    const taskStartTime = (0, _moment.default)().utc().toISOString();
    log('running task');
    const updatedState = {
      lastExecutionTimestamp: taskStartTime,
      namespace: state.namespace,
      runs: state.runs + 1
    };
    if (taskId !== getTaskId(state.namespace)) {
      log('outdated task; exiting');
      return {
        state: updatedState
      };
    }
    const entityTypes = (0, _united_entity_definitions.getAvailableEntityTypes)();
    for (const entityType of entityTypes) {
      const start = Date.now();
      debugLog(`executing field retention enrich policy for ${entityType}`);
      try {
        const {
          executed
        } = await executeEnrichPolicy(state.namespace, entityType);
        if (!executed) {
          debugLog(`Field retention enrich policy for ${entityType} does not exist`);
        } else {
          log(`Executed field retention enrich policy for ${entityType} in ${Date.now() - start}ms`);
        }
      } catch (e) {
        log(`error executing field retention enrich policy for ${entityType}: ${e.message}`);
      }
    }
    const taskCompletionTime = (0, _moment.default)().utc().toISOString();
    const taskDurationInSeconds = (0, _moment.default)(taskCompletionTime).diff((0, _moment.default)(taskStartTime), 'seconds');
    log(`task run completed in ${taskDurationInSeconds} seconds`);
    telemetry.reportEvent(_events.FIELD_RETENTION_ENRICH_POLICY_EXECUTION_EVENT.eventType, {
      duration: taskDurationInSeconds,
      interval: _constants.INTERVAL
    });

    // Track entity store usage
    const indices = entityTypes.map(entityType => (0, _utils.getEntitiesIndexName)(entityType, state.namespace));
    const storeSize = await getStoreSize(indices);
    telemetry.reportEvent(_events.ENTITY_STORE_USAGE_EVENT.eventType, {
      storeSize
    });
    return {
      state: updatedState
    };
  } catch (e) {
    logger.error(`[Entity Store] [task ${taskId}]: error running task, received ${e.message}`);
    throw e;
  }
};
exports.runTask = runTask;
const createTaskRunnerFactory = ({
  logger,
  telemetry,
  executeEnrichPolicy,
  getStoreSize
}) => ({
  taskInstance
}) => {
  let cancelled = false;
  const isCancelled = () => cancelled;
  return {
    run: async () => runTask({
      executeEnrichPolicy,
      getStoreSize,
      isCancelled,
      logger,
      taskInstance,
      telemetry
    }),
    cancel: async () => {
      cancelled = true;
    }
  };
};