"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RuleMigrationsDataRulesClient = void 0;
var _constants = require("../../../../../common/siem_migrations/constants");
var _rule_migrations_data_base_client = require("./rule_migrations_data_base_client");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

/* BULK_MAX_SIZE defines the number to break down the bulk operations by.
 * The 500 number was chosen as a reasonable number to avoid large payloads. It can be adjusted if needed.
 */
const BULK_MAX_SIZE = 500;
class RuleMigrationsDataRulesClient extends _rule_migrations_data_base_client.RuleMigrationsDataBaseClient {
  /** Indexes an array of rule migrations to be processed */
  async create(ruleMigrations) {
    const index = await this.getIndexName();
    let ruleMigrationsSlice;
    const createdAt = new Date().toISOString();
    while ((ruleMigrationsSlice = ruleMigrations.splice(0, BULK_MAX_SIZE)).length) {
      await this.esClient.bulk({
        refresh: 'wait_for',
        operations: ruleMigrationsSlice.flatMap(ruleMigration => [{
          create: {
            _index: index
          }
        }, {
          ...ruleMigration,
          '@timestamp': createdAt,
          status: _constants.SiemMigrationStatus.PENDING,
          created_by: this.username,
          updated_by: this.username,
          updated_at: createdAt
        }])
      }).catch(error => {
        this.logger.error(`Error creating rule migrations: ${error.message}`);
        throw error;
      });
    }
  }

  /** Updates an array of rule migrations to be processed */
  async update(ruleMigrations) {
    const index = await this.getIndexName();
    let ruleMigrationsSlice;
    const updatedAt = new Date().toISOString();
    while ((ruleMigrationsSlice = ruleMigrations.splice(0, BULK_MAX_SIZE)).length) {
      await this.esClient.bulk({
        refresh: 'wait_for',
        operations: ruleMigrationsSlice.flatMap(ruleMigration => {
          const {
            id,
            ...rest
          } = ruleMigration;
          return [{
            update: {
              _index: index,
              _id: id
            }
          }, {
            doc: {
              ...rest,
              updated_by: this.username,
              updated_at: updatedAt
            }
          }];
        })
      }).catch(error => {
        this.logger.error(`Error updating rule migrations: ${error.message}`);
        throw error;
      });
    }
  }

  /** Retrieves an array of rule documents of a specific migrations */
  async get(migrationId) {
    const index = await this.getIndexName();
    const query = this.getFilterQuery(migrationId);
    const storedRuleMigrations = await this.esClient.search({
      index,
      query,
      sort: '_doc'
    }).then(this.processResponseHits.bind(this)).catch(error => {
      this.logger.error(`Error searching rule migrations: ${error.message}`);
      throw error;
    });
    return storedRuleMigrations;
  }

  /**
   * Retrieves `pending` rule migrations with the provided id and updates their status to `processing`.
   * This operation is not atomic at migration level:
   * - Multiple tasks can process different migrations simultaneously.
   * - Multiple tasks should not process the same migration simultaneously.
   */
  async takePending(migrationId, size) {
    const index = await this.getIndexName();
    const query = this.getFilterQuery(migrationId, _constants.SiemMigrationStatus.PENDING);
    const storedRuleMigrations = await this.esClient.search({
      index,
      query,
      sort: '_doc',
      size
    }).then(response => this.processResponseHits(response, {
      status: _constants.SiemMigrationStatus.PROCESSING
    })).catch(error => {
      this.logger.error(`Error searching rule migrations: ${error.message}`);
      throw error;
    });
    await this.esClient.bulk({
      refresh: 'wait_for',
      operations: storedRuleMigrations.flatMap(({
        id,
        status
      }) => [{
        update: {
          _id: id,
          _index: index
        }
      }, {
        doc: {
          status,
          updated_by: this.username,
          updated_at: new Date().toISOString()
        }
      }])
    }).catch(error => {
      this.logger.error(`Error updating for rule migrations status to processing: ${error.message}`);
      throw error;
    });
    return storedRuleMigrations;
  }

  /** Updates one rule migration with the provided data and sets the status to `completed` */
  async saveCompleted({
    id,
    ...ruleMigration
  }) {
    const index = await this.getIndexName();
    const doc = {
      ...ruleMigration,
      status: _constants.SiemMigrationStatus.COMPLETED,
      updated_by: this.username,
      updated_at: new Date().toISOString()
    };
    await this.esClient.update({
      index,
      id,
      doc,
      refresh: 'wait_for'
    }).catch(error => {
      this.logger.error(`Error updating rule migration status to completed: ${error.message}`);
      throw error;
    });
  }

  /** Updates one rule migration with the provided data and sets the status to `failed` */
  async saveError({
    id,
    ...ruleMigration
  }) {
    const index = await this.getIndexName();
    const doc = {
      ...ruleMigration,
      status: _constants.SiemMigrationStatus.FAILED,
      updated_by: this.username,
      updated_at: new Date().toISOString()
    };
    await this.esClient.update({
      index,
      id,
      doc,
      refresh: 'wait_for'
    }).catch(error => {
      this.logger.error(`Error updating rule migration status to failed: ${error.message}`);
      throw error;
    });
  }

  /** Updates all the rule migration with the provided id with status `processing` back to `pending` */
  async releaseProcessing(migrationId) {
    return this.updateStatus(migrationId, _constants.SiemMigrationStatus.PROCESSING, _constants.SiemMigrationStatus.PENDING);
  }

  /** Updates all the rule migration with the provided id and with status `statusToQuery` to `statusToUpdate` */
  async updateStatus(migrationId, statusToQuery, statusToUpdate, {
    refresh = false
  } = {}) {
    const index = await this.getIndexName();
    const query = this.getFilterQuery(migrationId, statusToQuery);
    const script = {
      source: `ctx._source['status'] = '${statusToUpdate}'`
    };
    await this.esClient.updateByQuery({
      index,
      query,
      script,
      refresh
    }).catch(error => {
      this.logger.error(`Error updating rule migrations status: ${error.message}`);
      throw error;
    });
  }

  /** Retrieves the stats for the rule migrations with the provided id */
  async getStats(migrationId) {
    var _result$aggregations, _doc_count, _doc_count2, _doc_count3, _doc_count4;
    const index = await this.getIndexName();
    const query = this.getFilterQuery(migrationId);
    const aggregations = {
      pending: {
        filter: {
          term: {
            status: _constants.SiemMigrationStatus.PENDING
          }
        }
      },
      processing: {
        filter: {
          term: {
            status: _constants.SiemMigrationStatus.PROCESSING
          }
        }
      },
      completed: {
        filter: {
          term: {
            status: _constants.SiemMigrationStatus.COMPLETED
          }
        }
      },
      failed: {
        filter: {
          term: {
            status: _constants.SiemMigrationStatus.FAILED
          }
        }
      },
      lastUpdatedAt: {
        max: {
          field: 'updated_at'
        }
      }
    };
    const result = await this.esClient.search({
      index,
      query,
      aggregations,
      _source: false
    }).catch(error => {
      this.logger.error(`Error getting rule migrations stats: ${error.message}`);
      throw error;
    });
    const {
      pending,
      processing,
      completed,
      lastUpdatedAt,
      failed
    } = (_result$aggregations = result.aggregations) !== null && _result$aggregations !== void 0 ? _result$aggregations : {};
    return {
      rules: {
        total: this.getTotalHits(result),
        pending: (_doc_count = pending === null || pending === void 0 ? void 0 : pending.doc_count) !== null && _doc_count !== void 0 ? _doc_count : 0,
        processing: (_doc_count2 = processing === null || processing === void 0 ? void 0 : processing.doc_count) !== null && _doc_count2 !== void 0 ? _doc_count2 : 0,
        completed: (_doc_count3 = completed === null || completed === void 0 ? void 0 : completed.doc_count) !== null && _doc_count3 !== void 0 ? _doc_count3 : 0,
        failed: (_doc_count4 = failed === null || failed === void 0 ? void 0 : failed.doc_count) !== null && _doc_count4 !== void 0 ? _doc_count4 : 0
      },
      last_updated_at: lastUpdatedAt === null || lastUpdatedAt === void 0 ? void 0 : lastUpdatedAt.value_as_string
    };
  }

  /** Retrieves the stats for all the rule migrations aggregated by migration id */
  async getAllStats() {
    var _result$aggregations2, _ref;
    const index = await this.getIndexName();
    const aggregations = {
      migrationIds: {
        terms: {
          field: 'migration_id'
        },
        aggregations: {
          pending: {
            filter: {
              term: {
                status: _constants.SiemMigrationStatus.PENDING
              }
            }
          },
          processing: {
            filter: {
              term: {
                status: _constants.SiemMigrationStatus.PROCESSING
              }
            }
          },
          completed: {
            filter: {
              term: {
                status: _constants.SiemMigrationStatus.COMPLETED
              }
            }
          },
          failed: {
            filter: {
              term: {
                status: _constants.SiemMigrationStatus.FAILED
              }
            }
          },
          lastUpdatedAt: {
            max: {
              field: 'updated_at'
            }
          }
        }
      }
    };
    const result = await this.esClient.search({
      index,
      aggregations,
      _source: false
    }).catch(error => {
      this.logger.error(`Error getting all rule migrations stats: ${error.message}`);
      throw error;
    });
    const migrationsAgg = (_result$aggregations2 = result.aggregations) === null || _result$aggregations2 === void 0 ? void 0 : _result$aggregations2.migrationIds;
    const buckets = (_ref = migrationsAgg === null || migrationsAgg === void 0 ? void 0 : migrationsAgg.buckets) !== null && _ref !== void 0 ? _ref : [];
    return buckets.map(bucket => {
      var _bucket$pending$doc_c, _bucket$pending, _bucket$processing$do, _bucket$processing, _bucket$completed$doc, _bucket$completed, _bucket$failed$doc_co, _bucket$failed, _bucket$lastUpdatedAt;
      return {
        migration_id: bucket.key,
        rules: {
          total: bucket.doc_count,
          pending: (_bucket$pending$doc_c = (_bucket$pending = bucket.pending) === null || _bucket$pending === void 0 ? void 0 : _bucket$pending.doc_count) !== null && _bucket$pending$doc_c !== void 0 ? _bucket$pending$doc_c : 0,
          processing: (_bucket$processing$do = (_bucket$processing = bucket.processing) === null || _bucket$processing === void 0 ? void 0 : _bucket$processing.doc_count) !== null && _bucket$processing$do !== void 0 ? _bucket$processing$do : 0,
          completed: (_bucket$completed$doc = (_bucket$completed = bucket.completed) === null || _bucket$completed === void 0 ? void 0 : _bucket$completed.doc_count) !== null && _bucket$completed$doc !== void 0 ? _bucket$completed$doc : 0,
          failed: (_bucket$failed$doc_co = (_bucket$failed = bucket.failed) === null || _bucket$failed === void 0 ? void 0 : _bucket$failed.doc_count) !== null && _bucket$failed$doc_co !== void 0 ? _bucket$failed$doc_co : 0
        },
        last_updated_at: (_bucket$lastUpdatedAt = bucket.lastUpdatedAt) === null || _bucket$lastUpdatedAt === void 0 ? void 0 : _bucket$lastUpdatedAt.value_as_string
      };
    });
  }
  getFilterQuery(migrationId, status) {
    const filter = [{
      term: {
        migration_id: migrationId
      }
    }];
    if (status) {
      if (Array.isArray(status)) {
        filter.push({
          terms: {
            status
          }
        });
      } else {
        filter.push({
          term: {
            status
          }
        });
      }
    }
    return {
      bool: {
        filter
      }
    };
  }
}
exports.RuleMigrationsDataRulesClient = RuleMigrationsDataRulesClient;