"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.registerRoutes = registerRoutes;
var _elasticsearch = require("@elastic/elasticsearch");
var _boom = require("@hapi/boom");
var _coreHttpServer = require("@kbn/core-http-server");
var _serverRouteRepositoryUtils = require("@kbn/server-route-repository-utils");
var _sseUtilsServer = require("@kbn/sse-utils-server");
var _zod = require("@kbn/zod");
var _lodash = require("lodash");
var _rxjs = require("rxjs");
var _validation_objects = require("./validation_objects");
var _validate_and_decode_params = require("./validate_and_decode_params");
var _make_zod_validation_object = require("./make_zod_validation_object");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the "Elastic License
 * 2.0", the "GNU Affero General Public License v3.0 only", and the "Server Side
 * Public License v 1"; you may not use this file except in compliance with, at
 * your election, the "Elastic License 2.0", the "GNU Affero General Public
 * License v3.0 only", or the "Server Side Public License, v 1".
 */

const CLIENT_CLOSED_REQUEST = {
  statusCode: 499,
  body: {
    message: 'Client closed request'
  }
};
function registerRoutes({
  core,
  repository,
  logger,
  dependencies
}) {
  const routes = Object.values(repository);
  const router = core.http.createRouter();
  routes.forEach(route => {
    const {
      params,
      endpoint,
      options,
      handler
    } = route;
    const {
      method,
      pathname,
      version
    } = (0, _serverRouteRepositoryUtils.parseEndpoint)(endpoint);
    const wrappedHandler = async (context, request, response) => {
      try {
        const validatedParams = (0, _validate_and_decode_params.validateAndDecodeParams)(request, params);
        const {
          aborted,
          result
        } = await Promise.race([handler({
          request,
          response,
          context,
          params: validatedParams,
          logger,
          ...dependencies
        }).then(value => {
          return {
            aborted: false,
            result: value
          };
        }), request.events.aborted$.toPromise().then(() => {
          return {
            aborted: true,
            result: undefined
          };
        })]);
        if (aborted) {
          return response.custom(CLIENT_CLOSED_REQUEST);
        }
        if ((0, _coreHttpServer.isKibanaResponse)(result)) {
          return result;
        } else if ((0, _rxjs.isObservable)(result)) {
          return response.ok({
            body: (0, _sseUtilsServer.observableIntoEventSourceStream)(result)
          });
        } else {
          const body = result || {};
          return response.ok({
            body
          });
        }
      } catch (error) {
        logger.error(error);
        const opts = {
          statusCode: 500,
          body: {
            message: error.message,
            attributes: {
              data: {}
            }
          }
        };
        if (error instanceof _elasticsearch.errors.RequestAbortedError) {
          return response.custom((0, _lodash.merge)(opts, CLIENT_CLOSED_REQUEST));
        }
        if ((0, _boom.isBoom)(error)) {
          opts.statusCode = error.output.statusCode;
          opts.body.attributes.data = error === null || error === void 0 ? void 0 : error.data;
        }
        return response.custom(opts);
      }
    };
    logger.debug(`Registering endpoint ${endpoint}`);
    let validationObject;
    if (params === undefined) {
      validationObject = _validation_objects.noParamsValidationObject;
    } else if ((0, _zod.isZod)(params)) {
      validationObject = (0, _make_zod_validation_object.makeZodValidationObject)(params);
    } else {
      validationObject = _validation_objects.passThroughValidationObject;
    }
    const {
      security,
      ...restOptions
    } = options !== null && options !== void 0 ? options : {};
    if (!version) {
      router[method]({
        path: pathname,
        security,
        options: restOptions,
        validate: validationObject
      }, wrappedHandler);
    } else {
      router.versioned[method]({
        path: pathname,
        access: pathname.startsWith('/internal/') ? 'internal' : 'public',
        options: restOptions,
        security
      }).addVersion({
        version,
        validate: {
          request: validationObject
        }
      }, wrappedHandler);
    }
  });
}