"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.InferenceConnector = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _server = require("@kbn/actions-plugin/server");
var _stream = require("stream");
var _schema = require("../../../common/inference/schema");
var _constants = require("../../../common/inference/constants");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

class InferenceConnector extends _server.SubActionConnector {
  // Not using Axios
  getResponseErrorMessage(error) {
    throw new Error('Method not implemented.');
  }
  constructor(params) {
    super(params);
    (0, _defineProperty2.default)(this, "inferenceId", void 0);
    (0, _defineProperty2.default)(this, "taskType", void 0);
    this.provider = this.config.provider;
    this.taskType = this.config.taskType;
    this.inferenceId = this.config.inferenceId;
    this.logger = this.logger;
    this.connectorID = this.connector.id;
    this.connectorTokenClient = params.services.connectorTokenClient;
    this.registerSubActions();
  }
  registerSubActions() {
    this.registerSubAction({
      name: _constants.SUB_ACTION.COMPLETION,
      method: 'performApiCompletion',
      schema: _schema.ChatCompleteParamsSchema
    });
    this.registerSubAction({
      name: _constants.SUB_ACTION.RERANK,
      method: 'performApiRerank',
      schema: _schema.RerankParamsSchema
    });
    this.registerSubAction({
      name: _constants.SUB_ACTION.SPARSE_EMBEDDING,
      method: 'performApiSparseEmbedding',
      schema: _schema.SparseEmbeddingParamsSchema
    });
    this.registerSubAction({
      name: _constants.SUB_ACTION.TEXT_EMBEDDING,
      method: 'performApiTextEmbedding',
      schema: _schema.TextEmbeddingParamsSchema
    });
    this.registerSubAction({
      name: _constants.SUB_ACTION.COMPLETION_STREAM,
      method: 'performApiCompletionStream',
      schema: _schema.ChatCompleteParamsSchema
    });
  }

  /**
   * responsible for making a esClient inference method to perform chat completetion task endpoint and returning the service response data
   * @param input the text on which you want to perform the inference task.
   * @signal abort signal
   */
  async performApiCompletion({
    input,
    signal
  }) {
    const response = await this.performInferenceApi({
      inference_id: this.inferenceId,
      input,
      task_type: 'completion'
    }, false, signal);
    return response.completion;
  }

  /**
   * responsible for making a esClient inference method to rerank task endpoint and returning the response data
   * @param input the text on which you want to perform the inference task. input can be a single string or an array.
   * @query the search query text
   * @signal abort signal
   */
  async performApiRerank({
    input,
    query,
    signal
  }) {
    const response = await this.performInferenceApi({
      query,
      inference_id: this.inferenceId,
      input,
      task_type: 'rerank'
    }, false, signal);
    return response.rerank;
  }

  /**
   * responsible for making a esClient inference method sparse embedding task endpoint and returning the response data
   * @param input the text on which you want to perform the inference task.
   * @signal abort signal
   */
  async performApiSparseEmbedding({
    input,
    signal
  }) {
    const response = await this.performInferenceApi({
      inference_id: this.inferenceId,
      input,
      task_type: 'sparse_embedding'
    }, false, signal);
    return response.sparse_embedding;
  }

  /**
   * responsible for making a esClient inference method text embedding task endpoint and returning the response data
   * @param input the text on which you want to perform the inference task.
   * @signal abort signal
   */
  async performApiTextEmbedding({
    input,
    inputType,
    signal
  }) {
    const response = await this.performInferenceApi({
      inference_id: this.inferenceId,
      input,
      task_type: 'text_embedding',
      task_settings: {
        input_type: inputType
      }
    }, false, signal);
    return response.text_embedding;
  }

  /**
   * private generic method to avoid duplication esClient inference inference execute.
   * @param params InferenceInferenceRequest params.
   * @param asStream defines the type of the responce, regular or stream
   * @signal abort signal
   */
  async performInferenceApi(params, asStream = false, signal) {
    try {
      var _this$esClient;
      const response = await ((_this$esClient = this.esClient) === null || _this$esClient === void 0 ? void 0 : _this$esClient.inference.inference(params, {
        asStream,
        signal
      }));
      this.logger.info(`Perform Inference endpoint for task type "${this.taskType}" and inference id ${this.inferenceId}`);
      // TODO: const usageMetadata = response?.data?.usageMetadata;
      return response;
    } catch (err) {
      this.logger.error(`error perform inference endpoint API: ${err}`);
      throw err;
    }
  }
  async streamAPI({
    input,
    signal
  }) {
    const response = await this.performInferenceApi({
      inference_id: this.inferenceId,
      input,
      task_type: this.taskType
    }, true, signal);
    return response.pipe(new _stream.PassThrough());
  }

  /**
   *  takes input. It calls the streamApi method to make a
   *  request to the Inference API with the message. It then returns a Transform stream
   *  that pipes the response from the API through the transformToString function,
   *  which parses the proprietary response into a string of the response text alone
   * @param input A message to be sent to the API
   * @signal abort signal
   */
  async performApiCompletionStream({
    input,
    signal
  }) {
    const res = await this.streamAPI({
      input,
      signal
    });
    return res;
  }
}
exports.InferenceConnector = InferenceConnector;