"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.queryMonitorStatusAlert = queryMonitorStatusAlert;
var _pMap = _interopRequireDefault(require("p-map"));
var _times = _interopRequireDefault(require("lodash/times"));
var _lodash = require("lodash");
var _client_defaults = require("../../../../common/constants/client_defaults");
var _lib = require("../../../lib");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const DEFAULT_MAX_ES_BUCKET_SIZE = 10000;
const fields = ['@timestamp', 'summary', 'monitor', 'observer', 'config_id', 'error', 'agent', 'url', 'state', 'tags', 'service', 'labels'];
async function queryMonitorStatusAlert({
  esClient,
  monitorLocationIds,
  range,
  monitorQueryIds,
  monitorLocationsMap,
  numberOfChecks,
  includeRetests = true
}) {
  const idSize = Math.trunc(DEFAULT_MAX_ES_BUCKET_SIZE / monitorLocationIds.length || 1);
  const pageCount = Math.ceil(monitorQueryIds.length / idSize);
  const upConfigs = {};
  const downConfigs = {};
  await (0, _pMap.default)((0, _times.default)(pageCount), async i => {
    var _result$aggregations;
    const idsToQuery = monitorQueryIds.slice(i * idSize, i * idSize + idSize);
    const params = (0, _lib.createEsParams)({
      body: {
        size: 0,
        query: {
          bool: {
            filter: [...(includeRetests ? [_client_defaults.SUMMARY_FILTER] : [_client_defaults.FINAL_SUMMARY_FILTER]), (0, _client_defaults.getTimespanFilter)({
              from: range.from,
              to: range.to
            }), {
              terms: {
                'monitor.id': idsToQuery
              }
            }]
          }
        },
        aggs: {
          id: {
            terms: {
              field: 'monitor.id',
              size: idSize
            },
            aggs: {
              location: {
                terms: {
                  field: 'observer.name',
                  size: monitorLocationIds.length || 100
                },
                aggs: {
                  downChecks: {
                    filter: {
                      range: {
                        'summary.down': {
                          gte: '1'
                        }
                      }
                    }
                  },
                  totalChecks: {
                    top_hits: {
                      size: numberOfChecks,
                      sort: [{
                        '@timestamp': {
                          order: 'desc'
                        }
                      }],
                      _source: {
                        includes: fields
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    });
    if (monitorLocationIds.length > 0) {
      params.body.query.bool.filter.push({
        terms: {
          'observer.name': monitorLocationIds
        }
      });
    }
    const {
      body: result
    } = await esClient.search(params);
    (_result$aggregations = result.aggregations) === null || _result$aggregations === void 0 ? void 0 : _result$aggregations.id.buckets.forEach(({
      location,
      key: queryId
    }) => {
      const locationSummaries = location.buckets.map(({
        key: locationId,
        totalChecks,
        downChecks
      }) => {
        return {
          locationId,
          totalChecks,
          downChecks
        };
      });

      // discard any locations that are not in the monitorLocationsMap for the given monitor as well as those which are
      // in monitorLocationsMap but not in listOfLocations
      const monLocations = monitorLocationsMap === null || monitorLocationsMap === void 0 ? void 0 : monitorLocationsMap[queryId];
      const monQueriedLocations = (0, _lodash.intersection)(monLocations, monitorLocationIds);
      monQueriedLocations === null || monQueriedLocations === void 0 ? void 0 : monQueriedLocations.forEach(monLocationId => {
        const locationSummary = locationSummaries.find(summary => summary.locationId === monLocationId);
        if (locationSummary) {
          var _latestPing$summary$u, _latestPing$summary;
          const {
            totalChecks,
            downChecks
          } = locationSummary;
          const latestPing = totalChecks.hits.hits[0]._source;
          const downCount = downChecks.doc_count;
          const isLatestPingUp = ((_latestPing$summary$u = (_latestPing$summary = latestPing.summary) === null || _latestPing$summary === void 0 ? void 0 : _latestPing$summary.up) !== null && _latestPing$summary$u !== void 0 ? _latestPing$summary$u : 0) > 0;
          const configId = latestPing.config_id;
          const monitorQueryId = latestPing.monitor.id;
          const meta = {
            ping: latestPing,
            configId,
            monitorQueryId,
            locationId: monLocationId,
            timestamp: latestPing['@timestamp'],
            checks: {
              downWithinXChecks: totalChecks.hits.hits.reduce((acc, curr) => {
                var _curr$_source$summary;
                return acc + (((_curr$_source$summary = curr._source.summary.down) !== null && _curr$_source$summary !== void 0 ? _curr$_source$summary : 0) > 0 ? 1 : 0);
              }, 0),
              down: downCount
            },
            status: 'up'
          };
          if (downCount > 0) {
            downConfigs[`${configId}-${monLocationId}`] = {
              ...meta,
              status: 'down'
            };
          }
          if (isLatestPingUp) {
            upConfigs[`${configId}-${monLocationId}`] = {
              ...meta,
              status: 'up'
            };
          }
        }
      });
    });
  }, {
    concurrency: 5
  });
  return {
    upConfigs,
    downConfigs,
    enabledMonitorQueryIds: monitorQueryIds
  };
}