"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.queryMonitorStatus = queryMonitorStatus;
var _times = _interopRequireDefault(require("lodash/times"));
var _lodash = require("lodash");
var _alert_config = require("../../common/runtime_types/monitor_management/alert_config");
var _client_defaults = require("../../common/constants/client_defaults");
var _runtime_types = require("../../common/runtime_types");
var _lib = require("../lib");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const DEFAULT_MAX_ES_BUCKET_SIZE = 10000;
const fields = ['@timestamp', 'summary', 'monitor', 'observer', 'config_id', 'error', 'agent', 'url', 'state', 'tags'];
const getStatusQuery = ({
  idSize,
  idsToQuery,
  range,
  monitorLocationIds
}) => {
  const params = (0, _lib.createEsParams)({
    body: {
      size: 0,
      query: {
        bool: {
          filter: [_client_defaults.FINAL_SUMMARY_FILTER, {
            range: {
              '@timestamp': {
                gte: range.from,
                lte: range.to
              }
            }
          }, {
            terms: {
              'monitor.id': idsToQuery
            }
          }]
        }
      },
      aggs: {
        id: {
          terms: {
            field: 'monitor.id',
            size: idSize
          },
          aggs: {
            location: {
              terms: {
                field: 'observer.name',
                size: monitorLocationIds.length || 100
              },
              aggs: {
                status: {
                  top_hits: {
                    size: 1,
                    sort: [{
                      '@timestamp': {
                        order: 'desc'
                      }
                    }],
                    _source: {
                      includes: fields
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  });
  if (monitorLocationIds.length > 0) {
    var _params$body$query, _params$body$query$bo;
    (_params$body$query = params.body.query) === null || _params$body$query === void 0 ? void 0 : (_params$body$query$bo = _params$body$query.bool) === null || _params$body$query$bo === void 0 ? void 0 : _params$body$query$bo.filter.push({
      terms: {
        'observer.name': monitorLocationIds
      }
    });
  }
  return params;
};
async function queryMonitorStatus({
  esClient,
  monitorLocationIds,
  range,
  monitorQueryIds,
  monitorLocationsMap,
  monitorQueryIdToConfigIdMap,
  monitors
}) {
  const idSize = Math.trunc(DEFAULT_MAX_ES_BUCKET_SIZE / monitorLocationIds.length || 1);
  const pageCount = Math.ceil(monitorQueryIds.length / idSize);
  let up = 0;
  let down = 0;
  const upConfigs = {};
  const downConfigs = {};
  const monitorsWithoutData = new Map(Object.entries((0, _lodash.cloneDeep)(monitorLocationsMap)));
  const pendingConfigs = {};
  const disabledConfigs = {};
  monitors.filter(monitor => !monitor.attributes[_runtime_types.ConfigKey.ENABLED]).forEach(monitor => {
    var _monitor$attributes$C;
    const monitorQueryId = monitor.attributes[_runtime_types.ConfigKey.MONITOR_QUERY_ID];
    (_monitor$attributes$C = monitor.attributes[_runtime_types.ConfigKey.LOCATIONS]) === null || _monitor$attributes$C === void 0 ? void 0 : _monitor$attributes$C.forEach(location => {
      disabledConfigs[`${monitorQueryIdToConfigIdMap[monitorQueryId]}-${location.id}`] = {
        configId: `${monitorQueryIdToConfigIdMap[monitorQueryId]}`,
        monitorQueryId,
        status: 'disabled',
        locationId: location.id,
        locationLabel: location.label,
        ...getMonitorMeta(monitor)
      };
    });
  });
  const queries = (0, _times.default)(pageCount).map(i => {
    const idsToQuery = monitorQueryIds.slice(i * idSize, i * idSize + idSize);
    return getStatusQuery({
      idSize,
      monitorLocationIds,
      range,
      idsToQuery
    }).body;
  });
  if (queries.length) {
    const {
      responses
    } = await esClient.msearch(queries, 'getCurrentStatusOverview');
    responses.forEach(result => {
      var _result$aggregations;
      (_result$aggregations = result.aggregations) === null || _result$aggregations === void 0 ? void 0 : _result$aggregations.id.buckets.forEach(({
        location,
        key: queryId
      }) => {
        const locationSummaries = location.buckets.map(({
          status,
          key: locationName
        }) => {
          const ping = status.hits.hits[0]._source;
          return {
            location: locationName,
            ping
          };
        });
        const monitor = monitors.find(m => m.attributes[_runtime_types.ConfigKey.MONITOR_QUERY_ID] === queryId);

        // discard any locations that are not in the monitorLocationsMap for the given monitor as well as those which are
        // in monitorLocationsMap but not in listOfLocations
        const monLocations = monitorLocationsMap === null || monitorLocationsMap === void 0 ? void 0 : monitorLocationsMap[queryId];
        const monQueriedLocations = (0, _lodash.intersection)(monLocations, monitorLocationIds);
        monQueriedLocations === null || monQueriedLocations === void 0 ? void 0 : monQueriedLocations.forEach(monLocation => {
          const locationSummary = locationSummaries.find(summary => summary.location === monLocation);
          if (locationSummary) {
            var _ping$summary$down, _ping$summary, _ping$summary$up, _ping$summary2, _monitorsWithoutData$;
            const {
              ping
            } = locationSummary;
            const downCount = (_ping$summary$down = (_ping$summary = ping.summary) === null || _ping$summary === void 0 ? void 0 : _ping$summary.down) !== null && _ping$summary$down !== void 0 ? _ping$summary$down : 0;
            const upCount = (_ping$summary$up = (_ping$summary2 = ping.summary) === null || _ping$summary2 === void 0 ? void 0 : _ping$summary2.up) !== null && _ping$summary$up !== void 0 ? _ping$summary$up : 0;
            const configId = ping.config_id;
            const monitorQueryId = ping.monitor.id;
            const meta = {
              ping,
              configId,
              monitorQueryId,
              locationId: monLocation,
              timestamp: ping['@timestamp'],
              locationLabel: ping.observer.geo.name,
              ...getMonitorMeta(monitor)
            };
            if (downCount > 0) {
              down += 1;
              downConfigs[`${configId}-${monLocation}`] = {
                ...meta,
                status: 'down'
              };
            } else if (upCount > 0) {
              up += 1;
              upConfigs[`${configId}-${monLocation}`] = {
                ...meta,
                status: 'up'
              };
            }
            const monitorsMissingData = monitorsWithoutData.get(monitorQueryId) || [];
            monitorsWithoutData.set(monitorQueryId, monitorsMissingData === null || monitorsMissingData === void 0 ? void 0 : monitorsMissingData.filter(loc => loc !== monLocation));
            if (!((_monitorsWithoutData$ = monitorsWithoutData.get(monitorQueryId)) !== null && _monitorsWithoutData$ !== void 0 && _monitorsWithoutData$.length)) {
              monitorsWithoutData.delete(monitorQueryId);
            }
          }
        });
      });
    });
  }

  // identify the remaining monitors without data, to determine pending monitors
  for (const [queryId, locs] of monitorsWithoutData) {
    const monitor = monitors.find(m => m.attributes[_runtime_types.ConfigKey.MONITOR_QUERY_ID] === queryId);
    locs.forEach(loc => {
      var _monitor$attributes$C2, _monitor$attributes$C3;
      pendingConfigs[`${monitorQueryIdToConfigIdMap[queryId]}-${loc}`] = {
        configId: `${monitorQueryIdToConfigIdMap[queryId]}`,
        monitorQueryId: queryId,
        status: 'unknown',
        locationId: loc,
        locationLabel: (_monitor$attributes$C2 = monitor.attributes[_runtime_types.ConfigKey.LOCATIONS]) === null || _monitor$attributes$C2 === void 0 ? void 0 : (_monitor$attributes$C3 = _monitor$attributes$C2.find(location => location.id === loc)) === null || _monitor$attributes$C3 === void 0 ? void 0 : _monitor$attributes$C3.label,
        name: monitor.attributes[_runtime_types.ConfigKey.NAME],
        schedule: monitor.attributes[_runtime_types.ConfigKey.SCHEDULE].number,
        tags: monitor.attributes[_runtime_types.ConfigKey.TAGS],
        isEnabled: monitor.attributes[_runtime_types.ConfigKey.ENABLED],
        type: monitor.attributes[_runtime_types.ConfigKey.MONITOR_TYPE],
        projectId: monitor.attributes[_runtime_types.ConfigKey.PROJECT_ID],
        isStatusAlertEnabled: (0, _alert_config.isStatusEnabled)(monitor.attributes[_runtime_types.ConfigKey.ALERT_CONFIG]),
        updated_at: monitor.updated_at
      };
    });
  }
  return {
    up,
    down,
    pending: Object.values(pendingConfigs).length,
    upConfigs,
    downConfigs,
    pendingConfigs,
    enabledMonitorQueryIds: monitorQueryIds,
    disabledConfigs
  };
}
const getMonitorMeta = monitor => {
  var _monitor$namespaces;
  return {
    name: monitor.attributes[_runtime_types.ConfigKey.NAME],
    schedule: monitor.attributes[_runtime_types.ConfigKey.SCHEDULE].number,
    tags: monitor.attributes[_runtime_types.ConfigKey.TAGS],
    isEnabled: monitor.attributes[_runtime_types.ConfigKey.ENABLED],
    type: monitor.attributes[_runtime_types.ConfigKey.MONITOR_TYPE],
    projectId: monitor.attributes[_runtime_types.ConfigKey.PROJECT_ID],
    isStatusAlertEnabled: (0, _alert_config.isStatusEnabled)(monitor.attributes[_runtime_types.ConfigKey.ALERT_CONFIG]),
    updated_at: monitor.updated_at,
    spaceId: (_monitor$namespaces = monitor.namespaces) === null || _monitor$namespaces === void 0 ? void 0 : _monitor$namespaces[0]
  };
};