"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validateRouteSpaceName = exports.parseArrayFilters = exports.isMonitorsQueryFiltered = exports.getSavedObjectKqlFilter = exports.getMonitors = exports.getMonitorFilters = exports.findLocationItem = exports.SEARCH_FIELDS = exports.QuerySchema = exports.OverviewStatusSchema = void 0;
var _configSchema = require("@kbn/config-schema");
var _lodash = require("lodash");
var _esQuery = require("@kbn/es-query");
var _common = require("@kbn/spaces-plugin/common");
var _sort_field = require("../../common/runtime_types/monitor_management/sort_field");
var _get_all_locations = require("../synthetics_service/get_all_locations");
var _saved_objects = require("../../common/types/saved_objects");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const StringOrArraySchema = _configSchema.schema.maybe(_configSchema.schema.oneOf([_configSchema.schema.string(), _configSchema.schema.arrayOf(_configSchema.schema.string())]));
const QuerySchema = exports.QuerySchema = _configSchema.schema.object({
  page: _configSchema.schema.maybe(_configSchema.schema.number()),
  perPage: _configSchema.schema.maybe(_configSchema.schema.number()),
  sortField: _sort_field.MonitorSortFieldSchema,
  sortOrder: _configSchema.schema.maybe(_configSchema.schema.oneOf([_configSchema.schema.literal('desc'), _configSchema.schema.literal('asc')])),
  query: _configSchema.schema.maybe(_configSchema.schema.string()),
  filter: _configSchema.schema.maybe(_configSchema.schema.string()),
  tags: StringOrArraySchema,
  monitorTypes: StringOrArraySchema,
  locations: StringOrArraySchema,
  projects: StringOrArraySchema,
  schedules: StringOrArraySchema,
  status: StringOrArraySchema,
  searchAfter: _configSchema.schema.maybe(_configSchema.schema.arrayOf(_configSchema.schema.string())),
  monitorQueryIds: StringOrArraySchema,
  internal: _configSchema.schema.maybe(_configSchema.schema.boolean({
    defaultValue: false
  })),
  showFromAllSpaces: _configSchema.schema.maybe(_configSchema.schema.boolean())
});
const OverviewStatusSchema = exports.OverviewStatusSchema = _configSchema.schema.object({
  query: _configSchema.schema.maybe(_configSchema.schema.string()),
  filter: _configSchema.schema.maybe(_configSchema.schema.string()),
  tags: StringOrArraySchema,
  monitorTypes: StringOrArraySchema,
  locations: StringOrArraySchema,
  projects: StringOrArraySchema,
  monitorQueryIds: StringOrArraySchema,
  schedules: StringOrArraySchema,
  status: StringOrArraySchema,
  scopeStatusByLocation: _configSchema.schema.maybe(_configSchema.schema.boolean()),
  showFromAllSpaces: _configSchema.schema.maybe(_configSchema.schema.boolean())
});
const SEARCH_FIELDS = exports.SEARCH_FIELDS = ['name', 'tags.text', 'locations.id.text', 'locations.label', 'urls', 'hosts', 'project_id.text'];
const getMonitors = async (context, {
  fields
} = {}) => {
  const {
    perPage = 50,
    page,
    sortField,
    sortOrder,
    query,
    tags,
    monitorTypes,
    locations,
    filter = '',
    searchAfter,
    projects,
    schedules,
    monitorQueryIds,
    showFromAllSpaces
  } = context.request.query;
  const {
    filtersStr
  } = await getMonitorFilters({
    filter,
    monitorTypes,
    tags,
    locations,
    projects,
    schedules,
    monitorQueryIds,
    context
  });
  return context.savedObjectsClient.find({
    type: _saved_objects.syntheticsMonitorType,
    perPage,
    page,
    sortField: parseMappingKey(sortField),
    sortOrder,
    searchFields: SEARCH_FIELDS,
    search: query,
    filter: filtersStr,
    searchAfter,
    fields,
    ...(showFromAllSpaces && {
      namespaces: ['*']
    })
  });
};
exports.getMonitors = getMonitors;
const getMonitorFilters = async data => {
  const {
    context,
    locations
  } = data;
  const locationFilter = await parseLocationFilter(context, locations);
  return parseArrayFilters({
    ...data,
    locationFilter
  });
};
exports.getMonitorFilters = getMonitorFilters;
const parseArrayFilters = ({
  tags,
  filter,
  configIds,
  projects,
  monitorTypes,
  schedules,
  monitorQueryIds,
  locationFilter
}) => {
  const filtersStr = [filter, getSavedObjectKqlFilter({
    field: 'tags',
    values: tags
  }), getSavedObjectKqlFilter({
    field: 'project_id',
    values: projects
  }), getSavedObjectKqlFilter({
    field: 'type',
    values: monitorTypes
  }), getSavedObjectKqlFilter({
    field: 'locations.id',
    values: locationFilter
  }), getSavedObjectKqlFilter({
    field: 'schedule.number',
    values: schedules
  }), getSavedObjectKqlFilter({
    field: 'id',
    values: monitorQueryIds
  }), getSavedObjectKqlFilter({
    field: 'config_id',
    values: configIds
  })].filter(f => !!f).join(' AND ');
  return {
    filtersStr,
    locationFilter
  };
};
exports.parseArrayFilters = parseArrayFilters;
const getSavedObjectKqlFilter = ({
  field,
  values,
  operator = 'OR',
  searchAtRoot = false
}) => {
  if (values === 'All' || Array.isArray(values) && values !== null && values !== void 0 && values.includes('All')) {
    return undefined;
  }
  if ((0, _lodash.isEmpty)(values) || !values) {
    return '';
  }
  let fieldKey = '';
  if (searchAtRoot) {
    fieldKey = `${field}`;
  } else {
    fieldKey = `${_saved_objects.monitorAttributes}.${field}`;
  }
  if (Array.isArray(values)) {
    return `${fieldKey}:(${values.map(value => `"${(0, _esQuery.escapeQuotes)(value)}"`).join(` ${operator} `)})`;
  }
  return `${fieldKey}:"${(0, _esQuery.escapeQuotes)(values)}"`;
};
exports.getSavedObjectKqlFilter = getSavedObjectKqlFilter;
const parseLocationFilter = async (context, locations) => {
  var _findLocationItem$id2, _findLocationItem2;
  if (!locations || (locations === null || locations === void 0 ? void 0 : locations.length) === 0) {
    return;
  }
  const {
    allLocations
  } = await (0, _get_all_locations.getAllLocations)(context);
  if (Array.isArray(locations)) {
    return locations.map(loc => {
      var _findLocationItem$id, _findLocationItem;
      return (_findLocationItem$id = (_findLocationItem = findLocationItem(loc, allLocations)) === null || _findLocationItem === void 0 ? void 0 : _findLocationItem.id) !== null && _findLocationItem$id !== void 0 ? _findLocationItem$id : '';
    }).filter(val => !!val);
  }
  return [(_findLocationItem$id2 = (_findLocationItem2 = findLocationItem(locations, allLocations)) === null || _findLocationItem2 === void 0 ? void 0 : _findLocationItem2.id) !== null && _findLocationItem$id2 !== void 0 ? _findLocationItem$id2 : ''];
};
const findLocationItem = (query, locations) => {
  return locations.find(({
    id,
    label
  }) => query === id || label === query);
};

/**
 * Returns whether the query is likely to return a subset of monitor objects.
 * Useful where `absoluteTotal` needs to be determined with a separate call
 * @param monitorQuery { MonitorsQuery }
 */
exports.findLocationItem = findLocationItem;
const isMonitorsQueryFiltered = monitorQuery => {
  const {
    query,
    tags,
    monitorTypes,
    locations,
    status,
    filter,
    projects,
    schedules,
    monitorQueryIds
  } = monitorQuery;
  return !!query || !!filter || !!(locations !== null && locations !== void 0 && locations.length) || !!(monitorTypes !== null && monitorTypes !== void 0 && monitorTypes.length) || !!(tags !== null && tags !== void 0 && tags.length) || !!(status !== null && status !== void 0 && status.length) || !!(projects !== null && projects !== void 0 && projects.length) || !!(schedules !== null && schedules !== void 0 && schedules.length) || !!(monitorQueryIds !== null && monitorQueryIds !== void 0 && monitorQueryIds.length);
};
exports.isMonitorsQueryFiltered = isMonitorsQueryFiltered;
function parseMappingKey(key) {
  switch (key) {
    case 'schedule.keyword':
      return 'schedule.number';
    case 'project_id.keyword':
      return 'project_id';
    default:
      return key;
  }
}
const validateRouteSpaceName = async routeContext => {
  const {
    spaceId,
    server,
    request,
    response
  } = routeContext;
  if (spaceId === _common.DEFAULT_SPACE_ID) {
    // default space is always valid
    return {
      spaceId: _common.DEFAULT_SPACE_ID
    };
  }
  try {
    var _server$spaces;
    await ((_server$spaces = server.spaces) === null || _server$spaces === void 0 ? void 0 : _server$spaces.spacesService.getActiveSpace(request));
  } catch (error) {
    var _error$output;
    if (((_error$output = error.output) === null || _error$output === void 0 ? void 0 : _error$output.statusCode) === 404) {
      return {
        spaceId,
        invalidResponse: response.notFound({
          body: {
            message: `Kibana space '${spaceId}' does not exist`
          }
        })
      };
    }
  }
  return {
    invalidResponse: undefined
  };
};
exports.validateRouteSpaceName = validateRouteSpaceName;