"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useDebouncedValue = void 0;
var _react = require("react");
var _lodash = require("lodash");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the "Elastic License
 * 2.0", the "GNU Affero General Public License v3.0 only", and the "Server Side
 * Public License v 1"; you may not use this file except in compliance with, at
 * your election, the "Elastic License 2.0", the "GNU Affero General Public
 * License v3.0 only", or the "Server Side Public License, v 1".
 */

const DEFAULT_TIMEOUT = 256;
/**
 * Debounces value changes and updates inputValue on root state changes if no debounced changes
 * are in flight because the user is currently modifying the value.
 *
 * * allowFalsyValue: update upstream with all falsy values but null or undefined
 * * wait: debounce timeout
 */

const useDebouncedValue = ({
  onChange,
  value,
  defaultValue
}, {
  allowFalsyValue,
  wait = DEFAULT_TIMEOUT
} = {
  wait: DEFAULT_TIMEOUT
}) => {
  const [inputValue, setInputValue] = (0, _react.useState)(value);
  const unflushedChanges = (0, _react.useRef)(false);
  const shouldUpdateWithFalsyValue = Boolean(allowFalsyValue);

  // Save the initial value
  const initialValue = (0, _react.useRef)(defaultValue !== null && defaultValue !== void 0 ? defaultValue : value);
  const flushChangesTimeout = (0, _react.useRef)();
  const onChangeDebounced = (0, _react.useMemo)(() => {
    const callback = (0, _lodash.debounce)(val => {
      onChange(val);
      // do not reset unflushed flag right away, wait a bit for upstream to pick it up
      flushChangesTimeout.current = setTimeout(() => {
        unflushedChanges.current = false;
      }, wait);
    }, wait);
    return val => {
      if (flushChangesTimeout.current) {
        clearTimeout(flushChangesTimeout.current);
      }
      unflushedChanges.current = true;
      callback(val);
    };
  }, [onChange, wait]);
  (0, _react.useEffect)(() => {
    if (!unflushedChanges.current && value !== inputValue) {
      setInputValue(value);
    }
  }, [value, inputValue]);
  const handleInputChange = val => {
    setInputValue(val);
    const valueToUpload = shouldUpdateWithFalsyValue ? val !== null && val !== void 0 ? val : initialValue.current : val || initialValue.current;
    onChangeDebounced(valueToUpload);
  };
  return {
    inputValue,
    handleInputChange,
    initialValue: initialValue.current
  };
};
exports.useDebouncedValue = useDebouncedValue;