"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createModifiedPrebuiltRuleAssets = void 0;
var _lodash = require("lodash");
var _with_security_span = require("../../../../../utils/with_security_span");
var _detection_engine = require("../../../../../../common/api/detection_engine");
var _convert_rule_to_diffable = require("../../../../../../common/detection_engine/prebuilt_rules/diff/convert_rule_to_diffable");
var _assert_pick_version_is_target = require("./assert_pick_version_is_target");
var _create_props_to_rule_type_map = require("./create_props_to_rule_type_map");
var _calculate_rule_fields_diff = require("../../logic/diff/calculation/calculate_rule_fields_diff");
var _convert_prebuilt_rule_asset_to_rule_response = require("../../../rule_management/logic/detection_rules_client/converters/convert_prebuilt_rule_asset_to_rule_response");
var _get_value_for_field = require("./get_value_for_field");
/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

const createModifiedPrebuiltRuleAssets = ({
  upgradeableRules,
  requestBody,
  prebuiltRulesCustomizationEnabled
}) => {
  return (0, _with_security_span.withSecuritySpanSync)(createModifiedPrebuiltRuleAssets.name, () => {
    const defaultPickVersion = prebuiltRulesCustomizationEnabled ? _detection_engine.PickVersionValuesEnum.MERGED : _detection_engine.PickVersionValuesEnum.TARGET;
    const {
      pick_version: globalPickVersion = defaultPickVersion,
      mode
    } = requestBody;
    const {
      modifiedPrebuiltRuleAssets,
      processingErrors
    } = upgradeableRules.reduce((processedRules, upgradeableRule) => {
      const targetRuleType = upgradeableRule.target.type;
      const ruleId = upgradeableRule.target.rule_id;
      const fieldNames = _create_props_to_rule_type_map.FIELD_NAMES_BY_RULE_TYPE_MAP.get(targetRuleType);
      try {
        if (fieldNames === undefined) {
          throw new Error(`Unexpected rule type: ${targetRuleType}`);
        }
        const {
          current,
          target
        } = upgradeableRule;
        if (current.type !== target.type) {
          (0, _assert_pick_version_is_target.assertPickVersionIsTarget)({
            ruleId,
            requestBody
          });
        }
        const calculatedRuleDiff = (0, _calculate_rule_fields_diff.calculateRuleFieldsDiff)({
          base_version: upgradeableRule.base ? (0, _convert_rule_to_diffable.convertRuleToDiffable)((0, _convert_prebuilt_rule_asset_to_rule_response.convertPrebuiltRuleAssetToRuleResponse)(upgradeableRule.base)) : _detection_engine.MissingVersion,
          current_version: (0, _convert_rule_to_diffable.convertRuleToDiffable)(upgradeableRule.current),
          target_version: (0, _convert_rule_to_diffable.convertRuleToDiffable)((0, _convert_prebuilt_rule_asset_to_rule_response.convertPrebuiltRuleAssetToRuleResponse)(upgradeableRule.target))
        });
        if (mode === 'ALL_RULES' && globalPickVersion === 'MERGED') {
          const fieldsWithConflicts = Object.keys(getFieldsDiffConflicts(calculatedRuleDiff));
          if (fieldsWithConflicts.length > 0) {
            // If the mode is ALL_RULES, no fields can be overriden to any other pick_version
            // than "MERGED", so throw an error for the fields that have conflicts.
            throw new Error(`Merge conflicts found in rule '${ruleId}' for fields: ${fieldsWithConflicts.join(', ')}. Please resolve the conflict manually or choose another value for 'pick_version'`);
          }
        }
        const modifiedPrebuiltRuleAsset = createModifiedPrebuiltRuleAsset({
          upgradeableRule,
          fieldNames,
          requestBody,
          globalPickVersion,
          calculatedRuleDiff
        });
        processedRules.modifiedPrebuiltRuleAssets.push(modifiedPrebuiltRuleAsset);
        return processedRules;
      } catch (err) {
        processedRules.processingErrors.push({
          error: err,
          item: {
            rule_id: ruleId
          }
        });
        return processedRules;
      }
    }, {
      modifiedPrebuiltRuleAssets: [],
      processingErrors: []
    });
    return {
      modifiedPrebuiltRuleAssets,
      processingErrors
    };
  });
};
exports.createModifiedPrebuiltRuleAssets = createModifiedPrebuiltRuleAssets;
function createModifiedPrebuiltRuleAsset({
  upgradeableRule,
  fieldNames,
  globalPickVersion,
  requestBody,
  calculatedRuleDiff
}) {
  const modifiedPrebuiltRuleAsset = {};
  for (const fieldName of fieldNames) {
    modifiedPrebuiltRuleAsset[fieldName] = (0, _get_value_for_field.getValueForField)({
      fieldName,
      upgradeableRule,
      globalPickVersion,
      requestBody,
      ruleFieldsDiff: calculatedRuleDiff
    });
  }
  return modifiedPrebuiltRuleAsset;
}
const getFieldsDiffConflicts = ruleFieldsDiff => (0, _lodash.pickBy)(ruleFieldsDiff, diff => {
  return diff.conflict !== 'NONE';
});